/*
 * Meridian Prime
 * Copyright (C) 2002 Jason Cline
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <stdio.h>
#include <io.h>
#include "global.h"
#include "dsp.h"
#include "ini.h"

#define MAX_DSP_DLL 255
#define LOWER(c) (c >= 'A' && c<= 'Z') ? c + ('a' - 'A'): c

//input plugin management structure
typedef struct tagDSPPLUGINS{
  DSPPLUGIN *tPlugin[MAX_DSP_DLL];
  INT8         szFilename[MAX_DSP_DLL][255];
  UINT32       nPluginsTotal;
}DSPPLUGINS;

static DSPPLUGINS tDSPPlugins;

//free the loaded dll's
UINT32 freeDSPDLLs()
{
UINT32 i;

  for(i=0; i<MAX_DSP_DLL; i++){
    if(tDSPPlugins.tPlugin[i]){
      if(tDSPPlugins.tPlugin[i]->dspShutdown) tDSPPlugins.tPlugin[i]->dspShutdown(NULL);
      FreeLibrary(tDSPPlugins.tPlugin[i]->hInstance);
    }  
    tDSPPlugins.tPlugin[i]=NULL;
  }
  tDSPPlugins.nPluginsTotal=0;
  return(1);
}

//load a dsp dll after it has been found
UINT32 loadDSPDLLs(HWND hWnd, INT8 *szFile)
{
HINSTANCE hInstance;
typedef   DSPPLUGIN *(WINAPI *PFNINPUT)(HWND);
PFNINPUT  fProc;

  //try to load the library
  hInstance=LoadLibrary(szFile);
  //exit out if it is no good
  if(hInstance==NULL) return(0);
  //now make sure it is a valid plugin
  fProc=(PFNINPUT)GetProcAddress(hInstance, "exportMeridianPrimeGetDSPModule");
  //exit if it isnt a valid plugin
  if(fProc==NULL){
    FreeLibrary(hInstance);
    return(0);
  }
  //seems good so lets do our thing now
  tDSPPlugins.tPlugin[tDSPPlugins.nPluginsTotal]=(*fProc)(hWnd);
  //now we fill in the gaps
  tDSPPlugins.tPlugin[tDSPPlugins.nPluginsTotal]->hInstance=hInstance;
  tDSPPlugins.tPlugin[tDSPPlugins.nPluginsTotal]->iniWrite=iniWrite;
  tDSPPlugins.tPlugin[tDSPPlugins.nPluginsTotal]->iniRead=iniRead;
  //now call the startup routine
  if(tDSPPlugins.tPlugin[tDSPPlugins.nPluginsTotal]->dspStartup) tDSPPlugins.tPlugin[tDSPPlugins.nPluginsTotal]->dspStartup(hWnd);
  return(1);
}

//scan for dsp dll's and then call the load routine
UINT32 scanDSPDLLs(HWND hWnd, INT8 *szDirectory)
{
INT8        szFile[256];
_finddata_t fFind;
INT32       hFile;

  //no plugins yet
  ZeroMemory(&tDSPPlugins, sizeof(tDSPPlugins));
  //scan the directory for input plugins
  sprintf(szFile, "%s\\plugins\\dsp_*.dll", szDirectory);
  hFile=_findfirst(szFile, &fFind);
  if(hFile != -1 ){
    //load the plugin
    sprintf(szFile, "%s\\plugins\\%s", szDirectory, fFind.name);
    sprintf(tDSPPlugins.szFilename[tDSPPlugins.nPluginsTotal], "%s", fFind.name);
    if(loadDSPDLLs(hWnd, szFile)) tDSPPlugins.nPluginsTotal++;
    while(!_findnext(hFile, &fFind)){
    //load the plugin
      sprintf(szFile, "%s\\plugins\\%s", szDirectory, fFind.name);
      sprintf(tDSPPlugins.szFilename[tDSPPlugins.nPluginsTotal], "%s", fFind.name);
      if(loadDSPDLLs(hWnd, szFile)) tDSPPlugins.nPluginsTotal++;
    }
    _findclose(hFile);
  }
  return(1);
}

//reset the dsp files
UINT32 resetDSPs()
{
UINT32 i;

  for(i=0; i<MAX_DSP_DLL; i++){
    if(tDSPPlugins.tPlugin[i] && tDSPPlugins.tPlugin[i]->resetDSP) tDSPPlugins.tPlugin[i]->resetDSP();
  }
  return(1);
}

//process dsp plugins
UINT32 processDSPs(UINT32 nSamplerate, UINT32 nBPS, INT8 *pBuffer, UINT32 nSize)
{
UINT32 i, j;

  //go through all 10 priority levels
  for(j=1; j<11; j++){
    for(i=0; i<MAX_DSP_DLL; i++){
      if(tDSPPlugins.tPlugin[i] && tDSPPlugins.tPlugin[i]->processAudio  && tDSPPlugins.tPlugin[i]->nEnabled && tDSPPlugins.tPlugin[i]->nPriority==j) tDSPPlugins.tPlugin[i]->processAudio(nSamplerate, nBPS, pBuffer, nSize);
    }
  }
  return(1);
}

//return a pointer to a dsp dll based on the filename
DSPPLUGIN *getDSPDLL(INT8 *szFilename)
{
UINT32 i;

  for(i=0; i<tDSPPlugins.nPluginsTotal; i++){
    if(strcmp(tDSPPlugins.szFilename[i], szFilename)==0) return (tDSPPlugins.tPlugin[i]);
  }
  return(NULL);
}
