/*
 * Meridian Prime
 * Copyright (C) 2002 Jason Cline
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * aINT32 with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * The code below was contributed by AcidDose with some modifications
 * by me to better fit in with the function naming scheme I want to
 * stick with.
 */

#include <windows.h>
#include "global.h"

INT32 eqBuf[64], eqFlags,
      eqScale[8] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
      eqFreq[8] =  {0xBC,0x9B,0x7D,0x62,0x49,0x32,0x1F,0x0E},
      eqDCR[2], lt_pan[512], eqAmp, eqPan, eqGain;


//calculates the panning
void eqCalcPanT()
{
 INT32 n,l,r; 
 n = 256;
 while (n--)
 { 
  r = n << 1;
  l = 0x1FF - r;
  if (l > 0xFF) l = 0xFF;
  if (r > 0xFF) r = 0xFF;
  lt_pan[n] = l;
  lt_pan[n|0x100] = r;
 }
 eqAmp = 0xFF;
 eqPan = 0x80;
 eqGain = 0x80;
 eqFlags |= 0x01;
}

//sets the values of the equalizer
void eqGive(UINT32 *nEqualizer, UINT32 n, INT32 amp, INT32 pan, INT32 gain)
{
 eqAmp = (amp*2.55f);
 eqPan = 0x80 + (pan*1.27f);
 eqGain = 0x80 + (gain*1.27f);

  if (!eqFlags & 0x01) { eqCalcPanT(); }
 while (n--)
 {
  eqScale[7-n] = (UINT32)(nEqualizer[n] * 2.55f) - 0x7F;
  if (eqScale[7-n] > 0) eqScale[7-n] <<= 2;
 }
}

void eqAmplify(INT8 *pBuffer, UINT32 Samples)
{
 INT32 lgain, rgain;
 INT16 *pBuf = (INT16 *)pBuffer;
 if (!eqFlags & 0x01) { eqCalcPanT(); }
 lgain = ((lt_pan[eqPan] * eqAmp) >> 8);
 rgain = ((lt_pan[eqPan|0x100] * eqAmp) >> 8);

 Samples >>= 2;
 while (Samples--)
 {
  pBuf[0] = (pBuf[0] * lgain) >> 8;
  pBuf[1] = (pBuf[1] * rgain) >> 8;
  pBuf += 2;
 }
}

//processes the equalizer
void eqProcess(INT8 *pBuffer, UINT32 Samples)
{
 INT32 x, y, z, mix[2], lgain, rgain;
 INT16 * pBuf;
 pBuf = (INT16 *) pBuffer;

 if (!eqFlags & 0x01) { eqCalcPanT(); }
 lgain = ((lt_pan[eqPan] * ((eqAmp * eqGain) >> 7)) >> 8);
 rgain = ((lt_pan[eqPan|0x100] * ((eqAmp * eqGain) >> 7)) >> 8);

 Samples >>= 2;
 while (Samples--)
 {
  mix[0] = 0;
  mix[1] = 0;
  x = 0;
  y = 0;

  eqBuf[x+0] += ((pBuf[0] - eqBuf[x+0]) * eqFreq[y]) >> 8;
  eqBuf[x+1] += ((pBuf[1] - eqBuf[x+1]) * eqFreq[y]) >> 8;
  mix[0] += ((pBuf[0]-eqBuf[x+0]) * eqScale[y]) >> 7;
  mix[1] += ((pBuf[1]-eqBuf[x+1]) * eqScale[y]) >> 7;

  z = 6;
  while (z--)
  {
   x += 2;
   y += 1;
   eqBuf[x+0] += ((eqBuf[x-2]-eqBuf[x+0]) * eqFreq[y]) >> 8;
   eqBuf[x+1] += ((eqBuf[x-1]-eqBuf[x+1]) * eqFreq[y]) >> 8;
   mix[0] += ((eqBuf[x-2]-eqBuf[x+0]) * eqScale[y]) >> 7;
   mix[1] += ((eqBuf[x-1]-eqBuf[x+1]) * eqScale[y]) >> 7;
  } 
   x += 2;
   y += 1;
   eqBuf[x+0] += ((eqBuf[x-2]-eqBuf[x+0]) * eqFreq[y]) >> 8;
   eqBuf[x+1] += ((eqBuf[x-1]-eqBuf[x+1]) * eqFreq[y]) >> 8;
   mix[0] += (eqBuf[x+0] * eqScale[y]) >> 7;
   mix[1] += (eqBuf[x+1] * eqScale[y]) >> 7;

  mix[0] += pBuf[0];
  mix[1] += pBuf[1];
  eqDCR[0] += ((mix[0] - eqDCR[0]) >> 8);
  eqDCR[1] += ((mix[1] - eqDCR[1]) >> 8);
  mix[0] = ((mix[0] - eqDCR[0]) * lgain) >> 8;
  mix[1] = ((mix[1] - eqDCR[1]) * rgain) >> 8;
  if (mix[0] > 0x7FF0) { mix[0] = 0x7FF0; } else if (mix[0] < -0x7FF0) { mix[0] = -0x7FF0; }
  if (mix[1] > 0x7FF0) { mix[1] = 0x7FF0; } else if (mix[1] < -0x7FF0) { mix[1] = -0x7FF0; }
  *pBuf++ = (INT16) mix[0];
  *pBuf++ = (INT16) mix[1];
 }
}
