/*
 * Meridian Prime
 * Copyright (C) 2002 Jason Cline
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <stdio.h>
#include <io.h>
#include "global.h"
#include "ini.h"
#include "input.h"
#include "audio.h"

#define MAX_INPUT_DLL 255
#define LOWER(c) (c >= 'A' && c<= 'Z') ? c + ('a' - 'A'): c

//input plugin management structure
typedef struct tagINPUTPLUGINS{
  INPUTPLUGIN *tPlugin[MAX_INPUT_DLL];
  INT8         szFilename[MAX_INPUT_DLL][255];
  UINT32       nPluginsTotal;
}INPUTPLUGINS;

static INPUTPLUGINS tInputPlugins;

//build the filter for the open file dialog
INT8 *getInputFilter()
{
static INT8  szFilter[2048];
INT8         szExt[2048];
INT8         szPart[3][2048];
INT8         szTempFilter[2048];
UINT32       i, j, nLen;
INT8        *nPos;

  ZeroMemory(szFilter, sizeof(szFilter));
  ZeroMemory(szExt, sizeof(szExt));
  for(i=0; i<MAX_INPUT_DLL; i++){
    if(tInputPlugins.tPlugin[i]){
      //convert the file filter for easier use
      nLen=strlen(tInputPlugins.tPlugin[i]->szFilter);
      strcpy(szTempFilter, tInputPlugins.tPlugin[i]->szFilter);
      j=0;
      while(j < nLen){
        if(szTempFilter[j]=='=') szTempFilter[j]='\0';
        j++;
      }
      //now build the filter
      nPos=szTempFilter;
      //file description
      strcpy(szPart[0], nPos);
      nPos+=strlen(nPos)+1;
      //file extensions
      strcpy(szPart[1], nPos);
      nPos+=strlen(nPos)+1;
      //add them to the filter list
      sprintf(szPart[2], "%s\n%s\n", szPart[0], szPart[1]);
      strcat(szFilter, szPart[2]);
      sprintf(szPart[2], "%s;", szPart[1]);
      strcat(szExt, szPart[2]);
    }
  }
  //finish up the filter by adding a filter for all types
  sprintf(szPart[2], "All Supported Formats\n%s\n%s\n", szExt, szFilter);
  strcpy(szFilter, szPart[2]);
  //convert it to the proper format
  nLen=strlen(szFilter);
  i=0;
  while(i < nLen){
    if(szFilter[i]=='\n') szFilter[i]='\0';
    i++;
  }
  return(szFilter);
}

//return a pointer to an input dll based on the filename
INPUTPLUGIN *getInputDLL(INT8 *szFilename)
{
UINT32 i;

  for(i=0; i<tInputPlugins.nPluginsTotal; i++){
    if(strcmp(tInputPlugins.szFilename[i], szFilename)==0) return (tInputPlugins.tPlugin[i]);
  }
  return(NULL);
}

//free the loaded dll's
UINT32 freeInputDLLs()
{
UINT32 i;

  for(i=0; i<MAX_INPUT_DLL; i++){
    if(tInputPlugins.tPlugin[i]){
      if(tInputPlugins.tPlugin[i]->inputShutdown) tInputPlugins.tPlugin[i]->inputShutdown(NULL);
      FreeLibrary(tInputPlugins.tPlugin[i]->hInstance);
    }  
    tInputPlugins.tPlugin[i]=NULL;
  }
  tInputPlugins.nPluginsTotal=0;
  return(1);
}

//load an input dll after it has been found
UINT32 loadInputDLLs(HWND hWnd, INT8 *szFile)
{
HINSTANCE hInstance;
typedef   INPUTPLUGIN *(WINAPI *PFNINPUT)(HWND);
PFNINPUT  fProc;

  //try to load the library
  hInstance=LoadLibrary(szFile);
  //exit out if it is no good
  if(hInstance==NULL) return(0);
  //now make sure it is a valid plugin
  fProc=(PFNINPUT)GetProcAddress(hInstance, "exportMeridianPrimeGetInModule");
  //exit if it isnt a valid plugin
  if(fProc==NULL){
    FreeLibrary(hInstance);
    return(0);
  }
  //seems good so lets do our thing now
  tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]=(*fProc)(hWnd);
  //now we fill in the gaps
  tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->hInstance=hInstance;
  tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->audioShutdown=audioShutdown;
  tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->audioStartup=audioStartup;
  tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->audioCanWrite=audioCanWrite;
  tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->audioWrite=audioWrite;
  tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->iniWrite=iniWrite;
  tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->iniRead=iniRead;
  //now call the startup routine
  if(tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->inputStartup) tInputPlugins.tPlugin[tInputPlugins.nPluginsTotal]->inputStartup(NULL);
  return(1);
}

//scan for input dll's and then call the load routine
UINT32 scanInputDLLs(HWND hWnd, INT8 *szDirectory)
{
INT8        szFile[256];
_finddata_t fFind;
INT32       hFile;

  //no plugins yet
  ZeroMemory(&tInputPlugins, sizeof(tInputPlugins));
  //scan the directory for input plugins
  sprintf(szFile, "%s\\plugins\\in_*.dll", szDirectory);
  hFile=_findfirst(szFile, &fFind);
  if(hFile != -1 ){
    //load the plugin
    sprintf(szFile, "%s\\plugins\\%s", szDirectory, fFind.name);
    sprintf(tInputPlugins.szFilename[tInputPlugins.nPluginsTotal], "%s", fFind.name);
    if(loadInputDLLs(hWnd, szFile)) tInputPlugins.nPluginsTotal++;
    while(!_findnext(hFile, &fFind)){
    //load the plugin
      sprintf(szFile, "%s\\plugins\\%s", szDirectory, fFind.name);
      sprintf(tInputPlugins.szFilename[tInputPlugins.nPluginsTotal], "%s", fFind.name);
      if(loadInputDLLs(hWnd, szFile)) tInputPlugins.nPluginsTotal++;
    }
    _findclose(hFile);
  }
  return(1);
}

//return the plugin that corresponds to the file extension passed
INPUTPLUGIN *getPlugin(INT8 *szFilename)
{
UINT32 i, j;
INT8 *pFile;
INT8  szFile[255];
INT8  szFilter[255];

  pFile=szFilename;
  //we only want the extention
  while(*(pFile)!='.') pFile++;
  sprintf(szFile, "*%s", pFile);
  //convert the extension to lowercase
  for(i=0; i<strlen(szFile); i++)
    szFile[i]=LOWER(szFile[i]);
  //now scan through the files
  for(j=0; j<tInputPlugins.nPluginsTotal; j++){
    //now copy and convert the filter to lowercase
    strcpy(szFilter, tInputPlugins.tPlugin[j]->szFilter);
    for(i=0; i<strlen(szFilter); i++)
      szFilter[i]=LOWER(szFilter[i]);
    //check for a match
    if(strstr(szFilter, szFile)) return(tInputPlugins.tPlugin[j]);
  }
  return(NULL);
}