/*
 * Meridian Prime
 * Copyright (C) 2002 Jason Cline
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <stdio.h>
#include <io.h>
#include <time.h>
#include <commctrl.h>
#include "resource.h"
#include "global.h"
#include "audio.h"
#include "equalizer.h"
#include "skins.h"
#include "ini.h"
#include "input.h"
#include "dsp.h"
#include "zip.h"

#define  TIMER_CLOCK   0x20
#define  TIMER_REDRAW  0x21
#define  TIMER_SCROLL  0x22

#define  ACTION_NONE   0x00
#define  ACTION_BUTTON 0x01
#define  ACTION_VOLUME 0x02
#define  ACTION_GAIN   0x03
#define  ACTION_PAN    0x04
#define  ACTION_EQUAL1 0x05
#define  ACTION_EQUAL2 0x06
#define  ACTION_EQUAL3 0x07
#define  ACTION_EQUAL4 0x08
#define  ACTION_EQUAL5 0x09
#define  ACTION_EQUAL6 0x0a
#define  ACTION_EQUAL7 0x0b
#define  ACTION_EQUAL8 0x0c

#define  ACTION_MOVE   0x0d

#define  EQ_STYLE_CLASSICAL   0x01
#define  EQ_STYLE_FULL_BASS   0x02
#define  EQ_STYLE_FULL_TREBLE 0x03
#define  EQ_STYLE_POP         0x04
#define  EQ_STYLE_TECHNO      0x05

#define  WINDOW_MAIN          0x01
#define  WINDOW_EQUALIZER     0x02
#define  WINDOW_PLAYLIST      0x03
#define  WINDOW_KEYPAD        0x04

//config structure
typedef struct tagCONFIG{
  INT8   szProgramDirectory[1024];
  INT8   szWorkingDirectory[1024];
  INT8   szLastSkin[1024];
  INT32  nMainXPos, nMainYPos;
  UINT32 nVolume;
  INT32  nPan;
  INT32  nGain;
  UINT32 nEqualizer[8];
  UINT32 nEqualizerOn;
  UINT32 nWindowShow;
  UINT32 nOnTop;
}CONFIG;

//button structure
typedef struct tagBUTTON{
  UINT32 nState;
}BUTTON;

//globals
UINT32       nActive=1;
UINT32       nTimer=0;
UINT32       nTrack=0;
UINT32       nScrollSong=0;
UINT32       nMouseButtonDown=0;
UINT32       nMouseButtonAction=ACTION_NONE;
UINT32       nConfigQuickSelect;
INT8         szSongTitle[255]="\0";
HMENU        hPopupMenu=NULL;
CONFIG       tConfig;
BUTTON       tButton[SKINEND];
HWND         hMainWnd=NULL, hEqWnd=NULL;
INPUTPLUGIN *tCurrentInput=NULL;

//print a formatted message to the screen
void Message(char *string, ...)
{
va_list arg;
char szBuffer[1024];

  va_start(arg, string);
  vsprintf(szBuffer, string, arg);
  va_end(arg);
  MessageBox(NULL, szBuffer, "Meridian Prime", MB_OK);
}

//update the play clock by 1 second
void CALLBACK TimerClock(HWND hWnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
  //inc timer by 1
  if(audioPlaying()) nTimer++;
}

//update the redraw the skin
void CALLBACK TimerRedraw(HWND hWnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
UINT32 i;
INT8   szText[255];

  //MAIN WINDOW
  //start drawing
  skinDrawBegin(hWnd);
  //draw background
  skinDrawBackground(hWnd);
  //draw buttons
  for(i=SKINMAINBUTTONSTART+1; i<SKINMAINBUTTONEND; i++)
    skinDrawButton(hWnd, i, tButton[i].nState);
  if(tConfig.nEqualizerOn) skinDrawButton(hWnd, SKINEQUALIZERBUTTON, 1);
  //draw volume
  skinDrawVolume(hWnd, tConfig.nVolume);
  //draw timer
  skinDrawTime(hWnd, nTimer);
  //draw current track
  skinDrawTrack(hWnd, nTrack);
  //draw song title
  if(nMouseButtonAction==ACTION_VOLUME){
    sprintf(szText, "Volume: %i%%", tConfig.nVolume);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_GAIN){
    sprintf(szText, "Gain: %i%%", tConfig.nGain);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_EQUAL1){
    sprintf(szText, "Equalizer Bar 1: %i%%", tConfig.nEqualizer[0]);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_EQUAL2){
    sprintf(szText, "Equalizer Bar 2: %i%%", tConfig.nEqualizer[1]);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_EQUAL3){
    sprintf(szText, "Equalizer Bar 3: %i%%", tConfig.nEqualizer[2]);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_EQUAL4){
    sprintf(szText, "Equalizer Bar 4: %i%%", tConfig.nEqualizer[3]);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_EQUAL5){
    sprintf(szText, "Equalizer Bar 5: %i%%", tConfig.nEqualizer[4]);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_EQUAL6){
    sprintf(szText, "Equalizer Bar 6: %i%%", tConfig.nEqualizer[5]);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_EQUAL7){
    sprintf(szText, "Equalizer Bar 7: %i%%", tConfig.nEqualizer[6]);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_EQUAL8){
    sprintf(szText, "Equalizer Bar 8: %i%%", tConfig.nEqualizer[7]);
    skinDrawSongTitle(hWnd, szText);
  }
  else if(nMouseButtonAction==ACTION_PAN){
    if(tConfig.nPan==0) sprintf(szText, "Balance: Center");
    else if(tConfig.nPan<0) sprintf(szText, "Balance: %i%% Left", -tConfig.nPan);
    else if(tConfig.nPan>0) sprintf(szText, "Balance: %i%% Right", tConfig.nPan);
    skinDrawSongTitle(hWnd, szText);
  }
  else skinDrawSongTitle(hWnd, szSongTitle);
  //draw equalizer
  for(i=0; i<8; i++)
    skinDrawEqualizer(hWnd, i, tConfig.nEqualizer[i]);
  //draw the pan
  skinDrawPan(hWnd, tConfig.nPan);
  //draw the gain
  skinDrawGain(hWnd, tConfig.nGain);
  //end drawing
  skinDrawEnd(hWnd);
}

//scroll the title if it is to long every 1/3 seconds
void CALLBACK TimerScrollTitle(HWND hWnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
INT8   c;
UINT32 i;
RECT   rRect;

  //convenient place to get the window positions
  GetWindowRect(hWnd, &rRect);
  tConfig.nMainXPos=rRect.left;
  tConfig.nMainYPos=rRect.top;
  //scroll the song
  if(nScrollSong && szSongTitle!=NULL){
    //save first letter
    c=szSongTitle[0];
    //move everthing left by 1
    for(i=0; i<strlen(szSongTitle)-1; i++){
      szSongTitle[i]=szSongTitle[i+1];
    }
    //stick first letter back in at the end
    szSongTitle[i]=c;
  }
}

//retrieve the song number
void getSongNumber(HWND hWnd)
{
  if(tCurrentInput==NULL) nTrack=0;
  if(tCurrentInput->getTrackNumber) nTrack=tCurrentInput->getTrackNumber(hWnd);
  else nTrack=0;
}

//retrieve the song title
void getSongTitle(HWND hWnd)
{
INT8         *szTitle;

  nScrollSong=0;
  if(tCurrentInput==NULL) szSongTitle[0]='\0';
  if(tCurrentInput->getTrackName) szTitle=tCurrentInput->getTrackName(hWnd);
  else szTitle=NULL;
  if(szTitle) strcpy(szSongTitle, szTitle);
  else szSongTitle[0]='\0';
  //check the song length
  if(strlen(szSongTitle) > skinCheckTitleLength()){
    //the song title is to long so we scroll it
    nScrollSong=1;
    //lengthen the tittle and put some asterisks at the end
    sprintf(szSongTitle, "%s *** ", szSongTitle);
  }
}

//process the load button
void processLoadButton(HWND hWnd, INT8 *szExternalFile)
{
static OPENFILENAME ofnFile;
UINT32       nRet;
static INT8  szFilename[256];

  //dialog load
  if(szExternalFile[0]=='\0'){
    //set up the structure
    ofnFile.lStructSize=sizeof(OPENFILENAME);
    ofnFile.hwndOwner=hWnd;
    ofnFile.hInstance=NULL;
    ofnFile.lpstrFilter=getInputFilter();
    ofnFile.lpstrCustomFilter=NULL;
    ofnFile.nMaxCustFilter=0;
    ofnFile.nFilterIndex=0;
    ofnFile.lpstrFile=szFilename;
    ofnFile.nMaxFile=_MAX_PATH;
    ofnFile.lpstrFileTitle="Open File";
    ofnFile.nMaxFileTitle=_MAX_FNAME + _MAX_EXT;
    ofnFile.lpstrInitialDir=tConfig.szWorkingDirectory;
    ofnFile.lpstrTitle=NULL;
    ofnFile.Flags=OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
    ofnFile.nFileOffset=0;
    ofnFile.nFileExtension=0;
    ofnFile.lpstrDefExt=NULL;
    ofnFile.lCustData=0;
    ofnFile.lpfnHook=NULL;
    ofnFile.lpTemplateName=NULL;
    //call the common dialog interface
    nRet=GetOpenFileName(&ofnFile);
  }
  //drag n' drop or command line load
  else{
    strcpy(szFilename, szExternalFile);
    nRet=1;
  }
  eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  if(nRet){
    //store the working directory
    GetCurrentDirectory(255, tConfig.szWorkingDirectory);
    //stop the music
    if(tCurrentInput!=NULL && tCurrentInput->processStop) tCurrentInput->processStop(hWnd);
    //now we determine which kind of file it was and pass it off to the
    //correct plugin.
    tCurrentInput=getPlugin(szFilename);
    if(tCurrentInput!=NULL){
      if(tCurrentInput->processLoad){
        tCurrentInput->processLoad(hWnd, szFilename);
        getSongTitle(hWnd);
        getSongNumber(hWnd);
        //reset the time
        nTimer=0;
      }
    }
  }
}

//process the play button
void processPlayButton(HWND hWnd)
{
  if(tCurrentInput==NULL) return;
  //set the volume
  eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  //reset the time
  nTimer=0;
  //make it play
  if(tCurrentInput!=NULL){
    if(tCurrentInput->processPlay) tCurrentInput->processPlay(hWnd);
  }
}

//process the pause button
void processPauseButton(HWND hWnd)
{
  if(tCurrentInput==NULL) return;
}

//process the previous button
void processPreviousButton(HWND hWnd)
{
  if(tCurrentInput==NULL) return;
  if(tCurrentInput->processPrev) tCurrentInput->processPrev(hWnd);
  getSongTitle(hWnd);
  getSongNumber(hWnd);
  //reset the time
  nTimer=0;
}

//process the next button
void processNextButton(HWND hWnd)
{
  if(tCurrentInput==NULL) return;
  if(tCurrentInput->processNext) tCurrentInput->processNext(hWnd);
  getSongTitle(hWnd);
  getSongNumber(hWnd);
  //reset the time
  nTimer=0;
}

//process the stop button
void processStopButton(HWND hWnd)
{
  if(tCurrentInput==NULL) return;
  //stop the music
  if(tCurrentInput->processStop) tCurrentInput->processStop(hWnd);
  //reset the time
  nTimer=0;
}

//process the track information button
void processTrackInfoButton(HWND hWnd)
{
  if(tCurrentInput==NULL) return;
  //stop the music
  if(tCurrentInput->processSongInfo){
    tCurrentInput->processSongInfo(hWnd);
    getSongTitle(hWnd);
  }
}

//process the playlist button
void processPlaylistButton(HWND hWnd)
{
}

//process the minimize button
void processMinimizeButton(HWND hWnd)
{
  //shrink the window based on user input
  ShowWindow(hWnd, SW_MINIMIZE);
}

//process the exit button
void processExitButton(HWND hWnd)
{
  //stop the music
  processStopButton(hWnd);
  //close the window
  SendMessage(hWnd, WM_CLOSE, 0, 0);
}

//handle the volume slider
void processVolumeSlider(HWND hWnd, UINT32 nX, UINT32 nY)
{
  //get the new volume
  tConfig.nVolume=skinCheckVolume(nX, nY);
  //set the new equalizer, volume, pan, and gain values
  eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
}

//handle the pan slider
void processPanSlider(HWND hWnd, UINT32 nX, UINT32 nY)
{
  //get the new pan
  tConfig.nPan=skinCheckPan(nX, nY);
  //set the new equalizer, volume, pan, and gain values
  eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
}

//handle the gain slider
void processGainSlider(HWND hWnd, UINT32 nX, UINT32 nY)
{
  //get the new gain
  tConfig.nGain=skinCheckGain(nX, nY);
  //set the new equalizer, volume, pan, and gain values
  eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
}

//handle the equalizer slider
void processEqualizerSlider(HWND hWnd, UINT32 nX, UINT32 nY, UINT32 nEq)
{
  //get the new equalizer position
  tConfig.nEqualizer[nEq]=skinCheckEqualizer(nX, nY, nEq);
  //set the new equalizer, volume, pan, and gain values
  eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
}

//handle the equalizer presets
void processEqualizerPresets(HWND hWnd, UINT32 iStyle)
{
  if(iStyle==EQ_STYLE_CLASSICAL){
    tConfig.nEqualizer[0]=50;
    tConfig.nEqualizer[1]=50;
    tConfig.nEqualizer[2]=50;
    tConfig.nEqualizer[3]=50;
    tConfig.nEqualizer[4]=50;
    tConfig.nEqualizer[5]=50;
    tConfig.nEqualizer[6]=30;
    tConfig.nEqualizer[7]=30;
    eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  }
  else if(iStyle==EQ_STYLE_FULL_BASS){
    tConfig.nEqualizer[0]=90;
    tConfig.nEqualizer[1]=90;
    tConfig.nEqualizer[2]=70;
    tConfig.nEqualizer[3]=55;
    tConfig.nEqualizer[4]=45;
    tConfig.nEqualizer[5]=30;
    tConfig.nEqualizer[6]=10;
    tConfig.nEqualizer[7]=10;
    eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  }
  else if(iStyle==EQ_STYLE_FULL_TREBLE){
    tConfig.nEqualizer[0]=10;
    tConfig.nEqualizer[1]=10;
    tConfig.nEqualizer[2]=30;
    tConfig.nEqualizer[3]=45;
    tConfig.nEqualizer[4]=55;
    tConfig.nEqualizer[5]=70;
    tConfig.nEqualizer[6]=90;
    tConfig.nEqualizer[7]=90;
    eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  }
  else if(iStyle==EQ_STYLE_POP){
    tConfig.nEqualizer[0]=55;
    tConfig.nEqualizer[1]=85;
    tConfig.nEqualizer[2]=80;
    tConfig.nEqualizer[3]=65;
    tConfig.nEqualizer[4]=50;
    tConfig.nEqualizer[5]=45;
    tConfig.nEqualizer[6]=55;
    tConfig.nEqualizer[7]=65;
    eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  }
  else if(iStyle==EQ_STYLE_TECHNO){
    tConfig.nEqualizer[0]=90;
    tConfig.nEqualizer[1]=80;
    tConfig.nEqualizer[2]=60;
    tConfig.nEqualizer[3]=35;
    tConfig.nEqualizer[4]=45;
    tConfig.nEqualizer[5]=60;
    tConfig.nEqualizer[6]=70;
    tConfig.nEqualizer[7]=65;
    eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  }
}

//handles the about window messages
BOOL CALLBACK AboutDialogWindowProc(HWND hWnd, UINT32 uMsg, WPARAM wParam, LPARAM lParam)
{
  switch(uMsg){
    case WM_INITDIALOG:
      return(1);
    case WM_CLOSE:
      EndDialog(hWnd, 0);
      return(1);
  }
  return(0);
}

//clears the setup window
void processSetupClearPage(HWND hWnd)
{
HWND   hWndChild;
UINT32 i=0;

/*
#define MP_SKIN_TEMPLATE_LIST           1010
#define MP_SKIN_TEMPLATE_DESC           1011
#define MP_INPUT_TEMPLATE_LIST          1012
#define MP_INPUT_TEMPLATE_CONFIG        1013
#define MP_INPUT_TEMPLATE_ABOUT         1014
#define MP_OUTPUT_TEMPLATE_LIST         1015
#define MP_OUTPUT_TEMPLATE_CONFIG       1016
#define MP_OUTPUT_TEMPLATE_ABOUT        1017
#define MP_DSP_TEMPLATE_LIST            1018
#define MP_DSP_TEMPLATE_CONFIG          1019
#define MP_DSP_TEMPLATE_ABOUT           1020
#define MP_DSP_TEMPLATE_ENABLE          1021
#define MP_VISUALIZATION_TEMPLATE_LIST  1022
#define MP_VISUALIZATION_TEMPLATE_CONFIG 1023
#define MP_VISUALIZATION_TEMPLATE_ABOUT 1024
#define MP_VISUALIZATION_TEMPLATE_ENABLE 1025
#define MP_OPTIONS_TEMPLATE_STATIC      1026
#define MP_PLUGINS_TEMPLATE_STATIC      1027
*/

  //clear all the items
  for(i=MP_SKIN_TEMPLATE_LIST;i<=MP_PLUGINS_TEMPLATE_STATIC;i++){
    hWndChild=GetDlgItem(hWnd, i);
    ShowWindow(hWndChild, SW_HIDE);
  }
}

//sets up the setup window for options config
void processSetupOptionsPage(HWND hWnd)
{
HWND hWndChild;
UINT32 i=0;

  processSetupClearPage(hWnd);
  //change section name and make buttons/lists/etc visible
  SetDlgItemText(hWnd, MP_TREESELECTIONNAME, "Options");
  for(i=MP_OPTIONS_TEMPLATE_STATIC;i<=MP_OPTIONS_TEMPLATE_STATIC;i++){
    hWndChild=GetDlgItem(hWnd, i);
    ShowWindow(hWndChild, SW_SHOWNORMAL);
  }
}

//sets up the setup window for plug-ins config
void processSetupPluginsPage(HWND hWnd)
{
HWND hWndChild;
UINT32 i=0;

  processSetupClearPage(hWnd);
  //change section name and make buttons/lists/etc visible
  SetDlgItemText(hWnd, MP_TREESELECTIONNAME, "Plug-ins");
  for(i=MP_PLUGINS_TEMPLATE_STATIC;i<=MP_PLUGINS_TEMPLATE_STATIC;i++){
    hWndChild=GetDlgItem(hWnd, i);
    ShowWindow(hWndChild, SW_SHOWNORMAL);
  }
}

//sets up the setup window for plug-ins/input config
void processSetupPluginsInputPage(HWND hWnd)
{
HWND        hWndChild;
UINT32      i=0;
INT8        szFile[256];
_finddata_t fFind;
INT32       hFile;

  processSetupClearPage(hWnd);
  //change section name and make buttons/lists/etc visible
  SetDlgItemText(hWnd, MP_TREESELECTIONNAME, "Input Plug-ins");
  for(i=MP_INPUT_TEMPLATE_LIST;i<=MP_INPUT_TEMPLATE_ABOUT;i++){
    hWndChild=GetDlgItem(hWnd, i);
    ShowWindow(hWndChild, SW_SHOWNORMAL);
  }

  //fill in list
  SendDlgItemMessage(hWnd, MP_INPUT_TEMPLATE_LIST, LB_RESETCONTENT, 0, 0);
  sprintf(szFile, "%s\\plugins\\in_*.dll", tConfig.szProgramDirectory);
  hFile=_findfirst(szFile, &fFind);
  if(hFile != -1 ){
    SendDlgItemMessage(hWnd, MP_INPUT_TEMPLATE_LIST, LB_ADDSTRING, 0, (LPARAM)fFind.name);
    while(!_findnext(hFile, &fFind)){
      SendDlgItemMessage(hWnd, MP_INPUT_TEMPLATE_LIST, LB_ADDSTRING, 0, (LPARAM)fFind.name);
    }
    _findclose(hFile);
  }

}

//sets up the setup window for plug-ins/output config
void processSetupPluginsOutputPage(HWND hWnd)
{
HWND        hWndChild;
UINT32      i=0;
INT8        szFile[256];
_finddata_t fFind;
INT32       hFile;

  processSetupClearPage(hWnd);
  //change section name and make buttons/lists/etc visible
  SetDlgItemText(hWnd, MP_TREESELECTIONNAME, "Output Plug-ins");
  for(i=MP_OUTPUT_TEMPLATE_LIST;i<=MP_OUTPUT_TEMPLATE_ABOUT;i++){
    hWndChild=GetDlgItem(hWnd, i);
    ShowWindow(hWndChild, SW_SHOWNORMAL);
  }
}

//sets up the setup window for plug-ins/dsp config
void processSetupPluginsDSPPage(HWND hWnd)
{
HWND        hWndChild;
UINT32      i=0;
INT8        szFile[256];
_finddata_t fFind;
INT32       hFile;

  processSetupClearPage(hWnd);
  //change section name and make buttons/lists/etc visible
  SetDlgItemText(hWnd, MP_TREESELECTIONNAME, "DSP/Effect Plug-ins");
  for(i=MP_DSP_TEMPLATE_LIST;i<=MP_DSP_TEMPLATE_ENABLE;i++){
    hWndChild=GetDlgItem(hWnd, i);
    ShowWindow(hWndChild, SW_SHOWNORMAL);
  }

  //fill in list
  SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_RESETCONTENT, 0, 0);
  sprintf(szFile, "%s\\plugins\\dsp_*.dll", tConfig.szProgramDirectory);
  hFile=_findfirst(szFile, &fFind);
  if(hFile != -1 ){
    SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_ADDSTRING, 0, (LPARAM)fFind.name);
    while(!_findnext(hFile, &fFind)){
      SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_ADDSTRING, 0, (LPARAM)fFind.name);
    }
    _findclose(hFile);
  }
}

//sets up the setup window for plug-ins/visualizations config
void processSetupPluginsVisualizationPage(HWND hWnd)
{
HWND        hWndChild;
UINT32      i=0;
INT8        szFile[256];
_finddata_t fFind;
INT32       hFile;

  processSetupClearPage(hWnd);
  //change section name and make buttons/lists/etc visible
  SetDlgItemText(hWnd, MP_TREESELECTIONNAME, "Visualization Plug-ins");
  for(i=MP_VISUALIZATION_TEMPLATE_LIST;i<=MP_VISUALIZATION_TEMPLATE_ENABLE;i++){
    hWndChild=GetDlgItem(hWnd, i);
    ShowWindow(hWndChild, SW_SHOWNORMAL);
  }
}

//sets up the setup window for skin config
void processSetupSkinsPage(HWND hWnd)
{
HWND        hWndChild;
UINT32      i=0;
INT8        szFile[256];
_finddata_t fFind;
INT32       hFile;

  processSetupClearPage(hWnd);
  //change section name and make buttons/lists/etc visible
  SetDlgItemText(hWnd, MP_TREESELECTIONNAME, "Skins");
  for(i=MP_SKIN_TEMPLATE_LIST;i<=MP_SKIN_TEMPLATE_DESC;i++){
    hWndChild=GetDlgItem(hWnd, i);
    ShowWindow(hWndChild, SW_SHOWNORMAL);
  }
  //fill in skins list
  SendDlgItemMessage(hWnd, MP_SKIN_TEMPLATE_LIST, LB_RESETCONTENT, 0, 0);
  sprintf(szFile, "%s\\skins\\*.zip", tConfig.szProgramDirectory);
  hFile=_findfirst(szFile, &fFind);
  if(hFile != -1 ){
    SendDlgItemMessage(hWnd, MP_SKIN_TEMPLATE_LIST, LB_ADDSTRING, 0, (LPARAM)fFind.name);
    while(!_findnext(hFile, &fFind)){
      SendDlgItemMessage(hWnd, MP_SKIN_TEMPLATE_LIST, LB_ADDSTRING, 0, (LPARAM)fFind.name);
    }
    _findclose(hFile);
  }
}

//handles the setup window messages
BOOL CALLBACK SetupDialogWindowProc(HWND hWnd, UINT32 uMsg, WPARAM wParam, LPARAM lParam)
{
HTREEITEM       hTreeParent, hTreeItem;
TV_INSERTSTRUCT tvInsert;
UINT32          nCtrl;
LPNMTREEVIEW    pnmtv;
TVITEM          tvItem;
INT8            szText[255];
HWND            hWndChild;
INT8            szFile[256], szDir[256];
zipEntry       *pEntries=NULL;
INT32           nEntries=0;
INT8           *pMem=NULL;
UINT32          i, nSelect;
INPUTPLUGIN    *pInputDLL;
DSPPLUGIN      *pDSPDLL;


  switch(uMsg){
    case WM_INITDIALOG:

      //options
      tvInsert.hParent=NULL;
      tvInsert.hInsertAfter=TVI_ROOT;
      tvInsert.item.mask=TVIF_TEXT;
      tvInsert.item.pszText="Options";
      hTreeParent=(HTREEITEM)SendDlgItemMessage(hWnd, MP_TREE, TVM_INSERTITEM, 0, (LPARAM)&tvInsert);
      //plugins
      tvInsert.hParent=NULL;
      tvInsert.hInsertAfter=TVI_ROOT;
      tvInsert.item.mask=TVIF_TEXT;
      tvInsert.item.pszText="Plug-ins";
      hTreeParent=(HTREEITEM)SendDlgItemMessage(hWnd, MP_TREE, TVM_INSERTITEM, 0, (LPARAM)&tvInsert);
      tvInsert.hParent=hTreeParent;
      tvInsert.hInsertAfter=TVI_LAST;
      tvInsert.item.mask=TVIF_TEXT;
      tvInsert.item.pszText="Input";
      SendDlgItemMessage(hWnd, MP_TREE, TVM_INSERTITEM, 0, (LPARAM)&tvInsert);
      tvInsert.hParent=hTreeParent;
      tvInsert.hInsertAfter=TVI_LAST;
      tvInsert.item.mask=TVIF_TEXT;
      tvInsert.item.pszText="Output";
      SendDlgItemMessage(hWnd, MP_TREE, TVM_INSERTITEM, 0, (LPARAM)&tvInsert);
      tvInsert.hParent=hTreeParent;
      tvInsert.hInsertAfter=TVI_LAST;
      tvInsert.item.mask=TVIF_TEXT;
      tvInsert.item.pszText="DSP/Effect";
      SendDlgItemMessage(hWnd, MP_TREE, TVM_INSERTITEM, 0, (LPARAM)&tvInsert);
      tvInsert.hParent=hTreeParent;
      tvInsert.hInsertAfter=TVI_LAST;
      tvInsert.item.mask=TVIF_TEXT;
      tvInsert.item.pszText="Visualization";
      SendDlgItemMessage(hWnd, MP_TREE, TVM_INSERTITEM, 0, (LPARAM)&tvInsert);
      SendDlgItemMessage(hWnd, MP_TREE, TVM_EXPAND, (WPARAM)TVE_EXPAND, (LPARAM)hTreeParent);
      //skins
      tvInsert.hParent=NULL;
      tvInsert.hInsertAfter=TVI_ROOT;
      tvInsert.item.mask=TVIF_TEXT;
      tvInsert.item.pszText="Skins";
      hTreeParent=(HTREEITEM)SendDlgItemMessage(hWnd, MP_TREE, TVM_INSERTITEM, 0, (LPARAM)&tvInsert);
      nSelect=SendDlgItemMessage(hWnd, MP_TREE, LB_FINDSTRINGEXACT, -1, (LPARAM)"Skins");
      if(nConfigQuickSelect) SendDlgItemMessage(hWnd, MP_TREE, TVM_SELECTITEM, TVGN_CARET, (LPARAM)hTreeParent);
      nConfigQuickSelect=0;
      return(1);
    case WM_NOTIFY:
      nCtrl=(UINT32)wParam;
      pnmtv=(LPNMTREEVIEW)lParam;
      //the selection has changed so we find out which page to show
      if(pnmtv->hdr.code==TVN_SELCHANGED){
        hTreeItem=pnmtv->itemNew.hItem;
        ZeroMemory(&tvItem, sizeof(tvItem));
        tvItem.mask=TVIF_TEXT;
        tvItem.pszText=szText;
        tvItem.cchTextMax=256;
        tvItem.hItem=hTreeItem;
        //get the information about which item was selected
        SendDlgItemMessage(hWnd, MP_TREE, TVM_GETITEM, TVGN_CARET, (LPARAM)&tvItem);
        //now we find out which one was selected
        if(strcmp(tvItem.pszText,"Options")==0){
          processSetupOptionsPage(hWnd);
        }
        else if(strcmp(tvItem.pszText,"Plug-ins")==0){
          processSetupPluginsPage(hWnd);
        }
        else if(strcmp(tvItem.pszText,"Input")==0){
          processSetupPluginsInputPage(hWnd);
        }
        else if(strcmp(tvItem.pszText,"Output")==0){
          processSetupPluginsOutputPage(hWnd);
        }
        else if(strcmp(tvItem.pszText,"DSP/Effect")==0){
          processSetupPluginsDSPPage(hWnd);
        }
        else if(strcmp(tvItem.pszText,"Visualization")==0){
          processSetupPluginsVisualizationPage(hWnd);
        }
        else if(strcmp(tvItem.pszText,"Skins")==0){
          processSetupSkinsPage(hWnd);
        }
      }
      return(1);
    case WM_COMMAND:
      switch(wParam){
        //config an input DLL
        case MP_INPUT_TEMPLATE_CONFIG:
          //find out which input dll is selected
          nSelect=SendDlgItemMessage(hWnd, MP_INPUT_TEMPLATE_LIST, LB_GETCURSEL, 0, 0);
          SendDlgItemMessage(hWnd, MP_INPUT_TEMPLATE_LIST, LB_GETTEXT, nSelect, (LPARAM)szFile);
          //get the input dll and call the config routine if it has one
          pInputDLL=getInputDLL(szFile);
          if(pInputDLL==NULL || pInputDLL->processConfig==NULL) return(1);
          pInputDLL->processConfig(hWnd);
          return(1);
        //learn about an input DLL
        case MP_INPUT_TEMPLATE_ABOUT:
          //find out which input dll is selected
          nSelect=SendDlgItemMessage(hWnd, MP_INPUT_TEMPLATE_LIST, LB_GETCURSEL, 0, 0);
          SendDlgItemMessage(hWnd, MP_INPUT_TEMPLATE_LIST, LB_GETTEXT, nSelect, (LPARAM)szFile);
          //get the input dll and call the about routine if it has one
          pInputDLL=getInputDLL(szFile);
          if(pInputDLL==NULL || pInputDLL->processAbout==NULL) return(1);
          pInputDLL->processAbout(hWnd);
          return(1);


        //config a dsp DLL
        case MP_DSP_TEMPLATE_CONFIG:
          //find out which dsp dll is selected
          nSelect=SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_GETCURSEL, 0, 0);
          SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_GETTEXT, nSelect, (LPARAM)szFile);
          //get the input dll and call the config routine if it has one
          pDSPDLL=getDSPDLL(szFile);
          if(pDSPDLL==NULL || pDSPDLL->processConfig==NULL) return(1);
          pDSPDLL->processConfig(hWnd);
          return(1);
        //learn about a dsp DLL
        case MP_DSP_TEMPLATE_ABOUT:
          //find out which input dll is selected
          nSelect=SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_GETCURSEL, 0, 0);
          SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_GETTEXT, nSelect, (LPARAM)szFile);
          //get the input dll and call the about routine if it has one
          pDSPDLL=getDSPDLL(szFile);
          if(pDSPDLL==NULL || pDSPDLL->processAbout==NULL) return(1);
          pDSPDLL->processAbout(hWnd);
          return(1);
        //enable a dsp DLL
        case MP_DSP_TEMPLATE_ENABLE:
          //find out which input dll is selected
          nSelect=SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_GETCURSEL, 0, 0);
          SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_GETTEXT, nSelect, (LPARAM)szFile);
          //get the input dll and call the about routine if it has one
          pDSPDLL=getDSPDLL(szFile);
          if(pDSPDLL==NULL || pDSPDLL->processEnable==NULL) return(1);
          pDSPDLL->processEnable(hWnd);
          //set the enable/disable button status
          if(pDSPDLL->nEnabled) SetDlgItemText(hWnd, MP_DSP_TEMPLATE_ENABLE, "Disable");
          else SetDlgItemText(hWnd, MP_DSP_TEMPLATE_ENABLE, "Enable");
          return(1);
        //a new dsp was selected so we change the button status of Enable/Disable
        case (MP_DSP_TEMPLATE_LIST | 0x00010000):
          //find out which input dll is selected
          nSelect=SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_GETCURSEL, 0, 0);
          SendDlgItemMessage(hWnd, MP_DSP_TEMPLATE_LIST, LB_GETTEXT, nSelect, (LPARAM)szFile);
          //get the input dll and call the about routine if it has one
          pDSPDLL=getDSPDLL(szFile);
          if(pDSPDLL==NULL) return(1);
          if(pDSPDLL->nEnabled) SetDlgItemText(hWnd, MP_DSP_TEMPLATE_ENABLE, "Disable");
          else SetDlgItemText(hWnd, MP_DSP_TEMPLATE_ENABLE, "Enable");
          return(1);

        //a new skin was selected
        case (MP_SKIN_TEMPLATE_LIST | 0x00010000):
          //find out which skin was selected
          nSelect=SendDlgItemMessage(hWnd, MP_SKIN_TEMPLATE_LIST, LB_GETCURSEL, 0, 0);
          SendDlgItemMessage(hWnd, MP_SKIN_TEMPLATE_LIST, LB_GETTEXT, nSelect, (LPARAM)szFile);
          sprintf(szDir, "%s\\skins\\%s", tConfig.szProgramDirectory, szFile);
          hWndChild=GetDlgItem(hWnd, MP_SKIN_TEMPLATE_DESC);
          SetWindowText(hWndChild, "\0");
          //open the zip file, get the desc file if it exists, close the zip
          if(zipOpen(szDir)!=0){
            zipGetList(&pEntries, &nEntries);
            //find the desc file
            for(i=0;i<nEntries;i++){
              if(stricmp(pEntries[i].szName, "skin.txt")==0) break;
            }
            if(i<nEntries){
              pMem=(INT8 *)malloc(pEntries[i].nLen + 1);
              zipLoad((UINT8 *)pMem, pEntries[i].nLen, i);
              pMem[pEntries[i].nLen]='\0';
              //get the info and post it up
              SetWindowText(hWndChild, pMem);
              free(pMem);
            }
            zipClose();
	        zipFreeList(pEntries, nEntries);
          }
          //now load the skin
          skinFree();
          skinLoad(GetParent(hWnd), szDir, tConfig.nMainXPos, tConfig.nMainYPos);
          strcpy(tConfig.szLastSkin, szFile);
          return(1);
        //close the dialog box
        case MP_CLOSEBUTTON:
          EndDialog(hWnd, 0);
          return(1);
      }
      return(1);
    case WM_CLOSE:
      EndDialog(hWnd, 0);
      return(1);
  }
  return(0);
}

//handles all windows messaging for the main window
LRESULT CALLBACK WndMainProc(HWND hWnd, UINT32 uMsg, WPARAM wParam, LPARAM lParam)
{
POINTS  pMouse;
POINT   pPoint;
HMENU   hMenu;
UINT32  i, uMenuSelection;
HWND    hWndOld;
HDROP   hDrop;
INT8    szFile[255];

  switch(uMsg){
    //the window was just created
    case WM_CREATE:
      //create the timers
      SetTimer(hWnd, TIMER_CLOCK, 1000, TimerClock);
      SetTimer(hWnd, TIMER_REDRAW, 100, TimerRedraw);
      SetTimer(hWnd, TIMER_SCROLL, 300, TimerScrollTitle);
      return(0);

    //are we active or not
    case WM_ACTIVATE:
      if(!HIWORD(wParam)) nActive=1;
      else nActive=0;
      return(0);

    //drag n' drop
    case WM_DROPFILES:
      hDrop=(HDROP)wParam;
      //for now we only accept 1 file, late we will probably take as many as we can
      DragQueryFile(hDrop, 0, szFile, 255);
      DragFinish(hDrop);
      processLoadButton(hWnd, szFile);
      return(0);

    //mouse is moving
    case WM_MOUSEMOVE:
      pMouse=MAKEPOINTS(lParam);
      if(nMouseButtonAction==ACTION_BUTTON){
        for(i=SKINPLAYBUTTON; i<SKINVOLUMESLIDER; i++){
          if(skinCheckHit(pMouse.x, pMouse.y, i)) tButton[i].nState=1;
          else tButton[i].nState=0;
        }
        return(0);
      } 
      if(nMouseButtonAction==ACTION_EQUAL1){
        processEqualizerSlider(hWnd, pMouse.x, pMouse.y, 0);
        return(0);
      }
      if(nMouseButtonAction==ACTION_EQUAL2){
        processEqualizerSlider(hWnd, pMouse.x, pMouse.y, 1);
        return(0);
      }
      if(nMouseButtonAction==ACTION_EQUAL3){
        processEqualizerSlider(hWnd, pMouse.x, pMouse.y, 2);
        return(0);
      }
      if(nMouseButtonAction==ACTION_EQUAL4){
        processEqualizerSlider(hWnd, pMouse.x, pMouse.y, 3);
        return(0);
      }
      if(nMouseButtonAction==ACTION_EQUAL5){
        processEqualizerSlider(hWnd, pMouse.x, pMouse.y, 4);
        return(0);
      }
      if(nMouseButtonAction==ACTION_EQUAL6){
        processEqualizerSlider(hWnd, pMouse.x, pMouse.y, 5);
        return(0);
      }
      if(nMouseButtonAction==ACTION_EQUAL7){
        processEqualizerSlider(hWnd, pMouse.x, pMouse.y, 6);
        return(0);
      }
      if(nMouseButtonAction==ACTION_EQUAL8){
        processEqualizerSlider(hWnd, pMouse.x, pMouse.y, 7);
        return(0);
      }
      if(nMouseButtonAction==ACTION_PAN){
        processPanSlider(hWnd, pMouse.x, pMouse.y);
        return(0);
      }
      if(nMouseButtonAction==ACTION_GAIN){
        processGainSlider(hWnd, pMouse.x, pMouse.y);
        return(0);
      }
      if(nMouseButtonAction==ACTION_VOLUME){
        processVolumeSlider(hWnd, pMouse.x, pMouse.y);
        return(0);
      }
      return(0);

    //left mouse button down
    case WM_LBUTTONDOWN:
//      SetCapture(hWnd);
      pMouse=MAKEPOINTS(lParam);
      //clear buttons
      ZeroMemory(&tButton, sizeof(tButton));
      nMouseButtonDown=1;
      //test for a button hit
      for(i=SKINMAINBUTTONSTART+1; i<SKINMAINBUTTONEND; i++){
        if(skinCheckHit(pMouse.x, pMouse.y, i)){
          tButton[i].nState=1;
          nMouseButtonAction=ACTION_BUTTON;
          return(0);
        }
      }
      //test to see if we are moving the volume slider
      if(skinCheckHit(pMouse.x, pMouse.y, SKINVOLUMESLIDER)){
        nMouseButtonAction=ACTION_VOLUME;
        processVolumeSlider(hWnd, pMouse.x, pMouse.y);
        return(0);
      }
      //test to see if we are moving the equalizer slider
      for(i=0; i<8; i++){
        if(skinCheckHit(pMouse.x, pMouse.y, SKINEQUALIZER1 + i)){
          nMouseButtonAction=ACTION_EQUAL1 + i;
          processEqualizerSlider(hWnd, pMouse.x, pMouse.y, i);
          return(0);
        }
      }
      //test to see if we are moving the pan slider
      if(skinCheckHit(pMouse.x, pMouse.y, SKINPAN)){
        nMouseButtonAction=ACTION_PAN;
        processPanSlider(hWnd, pMouse.x, pMouse.y);
        return(0);
      }
      //test to see if we are moving the gain slider
      if(skinCheckHit(pMouse.x, pMouse.y, SKINGAIN)){
        nMouseButtonAction=ACTION_GAIN;
        processGainSlider(hWnd, pMouse.x, pMouse.y);
        return(0);
      }
      //test to see if we are moving the window
      if(skinCheckHit(pMouse.x, pMouse.y, SKINMAINMOVEAREA)){
        ReleaseCapture();
        nMouseButtonDown=0;
        nMouseButtonAction=ACTION_NONE;
        SendMessage(hWnd, WM_SYSCOMMAND, SC_MOVE | HTCLIENT, 0);
      }
      return(0);

    //left mouse button up
    case WM_LBUTTONUP:
      pMouse=MAKEPOINTS(lParam);
      //test for a button hit
      if(nMouseButtonAction==ACTION_BUTTON){
        for(i=SKINMAINBUTTONSTART+1; i<SKINMAINBUTTONEND; i++)
          if(skinCheckHit(pMouse.x, pMouse.y, i)) break;
        //process buttons if one was hit
        switch(i){
          case SKINLOADBUTTON:
            processLoadButton(hWnd, "");
            break;
          case SKINEXITBUTTON:
            processExitButton(hWnd);
            break;
          case SKINMINIMIZEBUTTON:
            processMinimizeButton(hWnd);
            break;
          case SKINPLAYBUTTON:
            processPlayButton(hWnd);
            break;
          case SKINPAUSEBUTTON:
            audioPause();
            break;
          case SKINSTOPBUTTON:
            processStopButton(hWnd);
            break;
          case SKINPREVBUTTON:
            processPreviousButton(hWnd);
            break;
          case SKINNEXTBUTTON:
            processNextButton(hWnd);
            break;
          case SKINTRACKINFOBUTTON:
            processTrackInfoButton(hWnd);
            break;
          case SKINEQUALIZERBUTTON:
            audioEqualizer();
            tConfig.nEqualizerOn=(tConfig.nEqualizerOn + 1) & 0x1;
            break;
        }
      }
      //clear buttons
      ZeroMemory(&tButton, sizeof(tButton));
      nMouseButtonDown=0;
      nMouseButtonAction=ACTION_NONE;
      ReleaseCapture();
      return(0);

    //right mouse button up
    case WM_RBUTTONUP:
      GetCursorPos(&pPoint);
      //now check the menu
      hMenu=GetSubMenu(hPopupMenu, 0);
      //configure menu
      if(tConfig.nOnTop) CheckMenuItem(hMenu, MP_POPUPMENU_ALWAYS_ON_TOP, MF_CHECKED);
      else CheckMenuItem(hMenu, MP_POPUPMENU_ALWAYS_ON_TOP, MF_UNCHECKED);
      if(tConfig.nEqualizerOn) CheckMenuItem(hMenu, MP_POPUPMENU_EQ_ENABLE, MF_CHECKED);
      else CheckMenuItem(hMenu, MP_POPUPMENU_EQ_ENABLE, MF_UNCHECKED);
      uMenuSelection=TrackPopupMenu(hMenu, TPM_TOPALIGN | TPM_LEFTALIGN | TPM_NONOTIFY | TPM_RETURNCMD | TPM_LEFTBUTTON, pPoint.x, pPoint.y, 0, hWnd, NULL);
      switch(uMenuSelection){
        case MP_POPUPMENU_MERIDIAN_PRIME:
          DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(MP_ABOUTDIALOG), hWnd, (DLGPROC)AboutDialogWindowProc);
          break;
        case MP_POPUPMENU_SKINBROWSER:
          nConfigQuickSelect=1;
          DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(MP_SETUPDIALOG), hWnd, (DLGPROC)SetupDialogWindowProc);
          break;
        case MP_POPUPMENU_LOAD:
          processLoadButton(hWnd, "");
          break;
        case MP_POPUPMENU_PLAY:
          processPlayButton(hWnd);
          break;
        case MP_POPUPMENU_PAUSE:
          audioPause();
          break;
        case MP_POPUPMENU_STOP:
          processStopButton(hWnd);
          break;
        case MP_POPUPMENU_PREVIOUS:
          processPreviousButton(hWnd);
          break;
        case MP_POPUPMENU_NEXT:
          processNextButton(hWnd);
          break;
        case MP_POPUPMENU_EXIT:
          processExitButton(hWnd);
          break;
        case MP_POPUPMENU_PREFERENCE:
          DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(MP_SETUPDIALOG), hWnd, (DLGPROC)SetupDialogWindowProc);
          break;
        case MP_POPUPMENU_EQUALIZER:
          break;

        case MP_POPUPMENU_EQ_CLASSICAL:
          processEqualizerPresets(hWnd, EQ_STYLE_CLASSICAL);
          break;
        case MP_POPUPMENU_EQ_FULL_BASS:
          processEqualizerPresets(hWnd, EQ_STYLE_FULL_BASS);
          break;
        case MP_POPUPMENU_EQ_FULL_TREBLE:
          processEqualizerPresets(hWnd, EQ_STYLE_FULL_TREBLE);
          break;
        case MP_POPUPMENU_EQ_POP:
          processEqualizerPresets(hWnd, EQ_STYLE_POP);
          break;
        case MP_POPUPMENU_EQ_TECHNO:
          processEqualizerPresets(hWnd, EQ_STYLE_TECHNO);
          break;
        case MP_POPUPMENU_EQ_ENABLE:
          audioEqualizer();
          tConfig.nEqualizerOn=(tConfig.nEqualizerOn + 1) & 0x1;
          break;

        case MP_POPUPMENU_ALWAYS_ON_TOP:
          tConfig.nOnTop=(tConfig.nOnTop + 1) % 2;
          if(tConfig.nOnTop){
            SetWindowPos(hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
            hWndOld=FindWindow("Meridian Prime Equalizer", NULL);
            SetWindowPos(hWndOld, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
          }
          else{
            SetWindowPos(hWnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
            hWndOld=FindWindow("Meridian Prime Equalizer", NULL);
            SetWindowPos(hWndOld, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
          }
          break;
      }
      return(0);

    //exit!
    case WM_CLOSE:
      PostQuitMessage(0);
      return(0);
  }
  return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

HWND InitiateWindow(HINSTANCE hInstance, LPSTR lpCmdLine)
{
WNDCLASS wcMainWindow;
HWND     hWnd, hWndOld;

  //only allow one instance of the program to run at a time
  CreateMutex(NULL, FALSE, "MERIDIANPRIME-045FA840-B10D-2G3-3436-006067709674");
  if(GetLastError() == ERROR_ALREADY_EXISTS || GetLastError() == ERROR_ACCESS_DENIED){
    //an instance is running so now we will send it the command line and exit out nicely
    hWndOld=FindWindow("Meridian Prime", NULL);
    if(hWndOld != NULL){
      if(lpCmdLine[0] != '\0'){
        //pass along any commandline options
      }
      //set already running instance to the foreground
      SetForegroundWindow(hWndOld);
      ShowWindow(hWndOld, SW_RESTORE);
    }
    return(NULL);
  }
  //create the main window
  ZeroMemory(&wcMainWindow, sizeof(wcMainWindow));
  wcMainWindow.style=CS_PARENTDC;
  wcMainWindow.lpszClassName="Meridian Prime";
  wcMainWindow.lpfnWndProc=(WNDPROC)WndMainProc;
  wcMainWindow.hInstance=hInstance;
  wcMainWindow.hCursor=LoadCursor(NULL, IDC_ARROW);
  wcMainWindow.hIcon=LoadIcon(NULL, IDI_WINLOGO);
  if(!RegisterClass(&wcMainWindow)) return(NULL);
  hWnd=CreateWindowEx(WS_EX_ACCEPTFILES, "Meridian Prime", "Meridian Prime", WS_POPUP, 10, 10, 200, 200, NULL, NULL, hInstance, NULL);
  ShowWindow(hWnd, SW_SHOWNORMAL);
  return(hWnd);
}

//read in the ini file
UINT32 fileReadINI(HWND hWnd)
{
INT8   *szData;
INT8   szFile[255];

  //clear config
  ZeroMemory(&tConfig, sizeof(tConfig));
  GetCurrentDirectory(255, tConfig.szProgramDirectory);
  //set up default values in case there is no ini file
  tConfig.nMainXPos=100;
  tConfig.nMainYPos=100;
  tConfig.nVolume=30;
  tConfig.nEqualizer[0]=50;
  tConfig.nEqualizer[1]=50;
  tConfig.nEqualizer[2]=50;
  tConfig.nEqualizer[3]=50;
  tConfig.nEqualizer[4]=50;
  tConfig.nEqualizer[5]=50;
  tConfig.nEqualizer[6]=50;
  tConfig.nEqualizer[7]=50;
  tConfig.nOnTop=0;
  tConfig.nWindowShow=WINDOW_MAIN;
  //read the ini file
  iniSetup(tConfig.szProgramDirectory);
  szData=iniRead("MERIDIAN PRIME", "LastSkin");         strcpy(tConfig.szLastSkin, szData);
  szData=iniRead("MERIDIAN PRIME", "WorkingDirectory"); strcpy(tConfig.szWorkingDirectory, szData);
  szData=iniRead("MERIDIAN PRIME", "PosX");             sscanf(szData, "%d", &tConfig.nMainXPos);
  szData=iniRead("MERIDIAN PRIME", "PosY");             sscanf(szData, "%d", &tConfig.nMainYPos);
  szData=iniRead("MERIDIAN PRIME", "Volume");           sscanf(szData, "%d", &tConfig.nVolume);
  szData=iniRead("MERIDIAN PRIME", "OnTop");            sscanf(szData, "%d", &tConfig.nOnTop);
  szData=iniRead("MERIDIAN PRIME", "EqValues");         sscanf(szData, "%d,%d,%d,%d,%d,%d,%d,%d", &tConfig.nEqualizer[0], &tConfig.nEqualizer[1], &tConfig.nEqualizer[2], &tConfig.nEqualizer[3], &tConfig.nEqualizer[4], &tConfig.nEqualizer[5], &tConfig.nEqualizer[6], &tConfig.nEqualizer[7]);
  szData=iniRead("MERIDIAN PRIME", "EqEnabled");        sscanf(szData, "%d", &tConfig.nEqualizerOn);
  szData=iniRead("MERIDIAN PRIME", "Pan");              sscanf(szData, "%d", &tConfig.nPan);
  szData=iniRead("MERIDIAN PRIME", "Gain");             sscanf(szData, "%d", &tConfig.nGain);
  //check for missing values
  if(tConfig.szWorkingDirectory[0]=='\0') GetCurrentDirectory(255, tConfig.szWorkingDirectory);
  if(tConfig.szLastSkin[0]=='\0') strcpy(tConfig.szLastSkin, "default.zip");
  //load the main skin
  sprintf(szFile, "%s\\skins\\%s", tConfig.szProgramDirectory, tConfig.szLastSkin);
  if(skinLoad(hWnd, szFile, tConfig.nMainXPos, tConfig.nMainYPos)==0) return(0);
  //position windows
  SetWindowPos(hWnd, HWND_NOTOPMOST, tConfig.nMainXPos, tConfig.nMainYPos, 0, 0, SWP_NOSIZE);
  //process config options
  if(tConfig.nOnTop) SetWindowPos(hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
  else SetWindowPos(hWnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
  eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  if(tConfig.nEqualizerOn) audioEqualizer();
  //set the new equalizer, volume, pan, and gain values
  eqGive(&tConfig.nEqualizer[0], 8, tConfig.nVolume, tConfig.nPan, tConfig.nGain);
  return(1);
}

//write out the config ini file
void fileWriteINI()
{
  iniSetup(tConfig.szProgramDirectory);
  iniWrite("MERIDIAN PRIME", "LastSkin", "%s", tConfig.szLastSkin);
  iniWrite("MERIDIAN PRIME", "WorkingDirectory", "%s", tConfig.szWorkingDirectory);
  iniWrite("MERIDIAN PRIME", "PosX", "%d", tConfig.nMainXPos);
  iniWrite("MERIDIAN PRIME", "PosY", "%d", tConfig.nMainYPos);
  iniWrite("MERIDIAN PRIME", "Volume", "%d", tConfig.nVolume);
  iniWrite("MERIDIAN PRIME", "OnTop", "%d", tConfig.nOnTop);
  iniWrite("MERIDIAN PRIME", "EqValues", "%d,%d,%d,%d,%d,%d,%d,%d", tConfig.nEqualizer[0], tConfig.nEqualizer[1], tConfig.nEqualizer[2], tConfig.nEqualizer[3], tConfig.nEqualizer[4], tConfig.nEqualizer[5], tConfig.nEqualizer[6], tConfig.nEqualizer[7]);
  iniWrite("MERIDIAN PRIME", "EqEnabled", "%d", tConfig.nEqualizerOn);
  iniWrite("MERIDIAN PRIME", "Pan", "%d", tConfig.nPan);
  iniWrite("MERIDIAN PRIME", "Gain", "%d", tConfig.nGain);
}

void programShutdown()
{
  //write the ini file
  fileWriteINI();
  //free the input DLL files
  freeInputDLLs();
  //free the dsp DLL files
  freeDSPDLLs();
  //unload the skin
  skinFree();
}

//program entry point
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
MSG      mMsg;
UINT32   iDone=0;
HWND     hWnd;

  InitCommonControls();
  //set up window
  if((hWnd=InitiateWindow(hInstance, lpCmdLine))==NULL) return(0);
  //read config ini flie
  if(fileReadINI(hWnd)==0){
    programShutdown();
    return(0);
  }
  //scan for input plugins
  if(scanInputDLLs(hWnd, tConfig.szProgramDirectory)==0){
    programShutdown();
    return(0);
  }
  //scan for dsp plugins
  if(scanDSPDLLs(hWnd, tConfig.szProgramDirectory)==0){
    programShutdown();
    return(0);
  }
  //load the config menu
  if((hPopupMenu=LoadMenu(hInstance, MAKEINTRESOURCE(MP_POPUPMENU)))==NULL){
    programShutdown();
    return(0);
  }
  //clear some structs
  ZeroMemory(&tButton, sizeof(tButton));
  //message loop
  while(!iDone){
    if(WaitMessage()){
      while(PeekMessage(&mMsg, NULL, 0, 0, PM_REMOVE)){
        if(mMsg.message==WM_QUIT) iDone=1;
        else{
          TranslateMessage(&mMsg);
          DispatchMessage(&mMsg);
        }
      }
    }
  }
  //shut the program down
  programShutdown();
  //destroy the config menu
  DestroyMenu(hPopupMenu);
  //exit the program
  return (mMsg.wParam);
}