/*
 * Meridian Prime
 * Copyright (C) 2002 Jason Cline
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <stdio.h>
#include "global.h"
#include "zip.h"

//a button or something
typedef struct tagLOCATION{
  //hit location
  INT32 x1, y1, w1, h1, x2, y2, w2, h2;
  //misc variables dependant on buttons
  INT32 a, b, c, d;
}LOCATION;

//skin struct
typedef struct tagSKIN{
  INT8     szINI[255];
  INT8     szBmpMainUp[255];
  INT8     szBmpMainDown[255];
  INT8     szBmpMainTextFont[255];
  INT8     szBmpMainTimeFont[255];
  INT8     szBmpMainTrackFont[255];

  COLORREF cTransparent;

  HBITMAP  hBmpMainUp;
  HBITMAP  hBmpMainDown;
  HBITMAP  hBmpMainTextFont;
  HBITMAP  hBmpMainTimeFont;
  HBITMAP  hBmpMainTrackFont;

  UINT32   nMainWidth;
  UINT32   nMainHeight;
  UINT32   nEqWidth;
  UINT32   nEqHeight;
  LOCATION tLocation[SKINEND];
}SKIN;

//ini fields
typedef struct tagASSOC{
  INT8 szField[255];
}ASSOC;

ASSOC tAssoc[SKINEND]={
  "Blank",
  "PlayButton",
  "StopButton",
  "PauseButton",
  "MinimizeButton",
  "ExitButton",
  "LoadButton",
  "NextButton",
  "PrevButton",
  "EqualizerButton",
  "PlaylistButton",
  "TrackInfo",
  "Blank",
  "VolumeSlider",
  "TitleText",
  "TrackText",
  "TimeText",
  "MainMoveArea",
  "Blank",
  "Gain",
  "Pan",
  "Eq1",
  "Eq2",
  "Eq3",
  "Eq4",
  "Eq5",
  "Eq6",
  "Eq7",
  "Eq8",
  "Blank",
  "Blank",
  "TransparentColor",
  "BmpMainUp",
  "BmpMainDown",
  "BmpMainTextFont",
  "BmpMainTimeFont",
  "BmpMainTrackFont",
};

//our skin
SKIN    tSkin;

//device contexts for drawing
HDC     hMainDC;
HDC     hDC;
HDC     hBmpDC;
HBITMAP hBackBuffer;

//begin the draw routine
void skinDrawBegin(HWND hWnd)
{
  //get device contexts
  hMainDC=GetDC(hWnd);
  hDC=CreateCompatibleDC(hMainDC);
  SelectObject(hDC, hBackBuffer);
  hBmpDC=CreateCompatibleDC(hMainDC);
}

//end the draw routine
void skinDrawEnd(HWND hWnd)
{
  //flip the backbuffer to the front
  BitBlt(hMainDC, 0, 0, tSkin.nMainWidth, tSkin.nMainHeight, hDC, 0, 0, SRCCOPY);
  //release device contexts
  DeleteDC(hDC);
  DeleteDC(hBmpDC);
  ReleaseDC(hWnd, hMainDC);
}

//draw the full background
void skinDrawBackground(HWND hWnd)
{
  //draw the main background
  SelectObject(hBmpDC, tSkin.hBmpMainUp);
  BitBlt(hDC, 0, 0, tSkin.nMainWidth, tSkin.nMainHeight, hBmpDC, 0, 0, SRCCOPY);
}

//draw the current track number
void skinDrawTrack(HWND hWnd, UINT32 nTrack)
{
INT8   szTrk[8];

  //convert to text
  sprintf(szTrk, "%03i", nTrack);
  SelectObject(hBmpDC, tSkin.hBmpMainTrackFont);
  //draw it!
  BitBlt(hDC, tSkin.tLocation[SKINTRACKTEXT].x1 + (tSkin.tLocation[SKINTRACKTEXT].w1 * 0), tSkin.tLocation[SKINTRACKTEXT].y1, tSkin.tLocation[SKINTRACKTEXT].w1, tSkin.tLocation[SKINTRACKTEXT].h1, hBmpDC, ((szTrk[0]-'0') * tSkin.tLocation[SKINTRACKTEXT].w1), 0, SRCCOPY);
  BitBlt(hDC, tSkin.tLocation[SKINTRACKTEXT].x1 + (tSkin.tLocation[SKINTRACKTEXT].w1 * 1), tSkin.tLocation[SKINTRACKTEXT].y1, tSkin.tLocation[SKINTRACKTEXT].w1, tSkin.tLocation[SKINTRACKTEXT].h1, hBmpDC, ((szTrk[1]-'0') * tSkin.tLocation[SKINTRACKTEXT].w1), 0, SRCCOPY);
  BitBlt(hDC, tSkin.tLocation[SKINTRACKTEXT].x1 + (tSkin.tLocation[SKINTRACKTEXT].w1 * 2), tSkin.tLocation[SKINTRACKTEXT].y1, tSkin.tLocation[SKINTRACKTEXT].w1, tSkin.tLocation[SKINTRACKTEXT].h1, hBmpDC, ((szTrk[2]-'0') * tSkin.tLocation[SKINTRACKTEXT].w1), 0, SRCCOPY);
}


//draw the time playing
void skinDrawTime(HWND hWnd, UINT32 nTime)
{
UINT32 nMin=nTime/60;
UINT32 nHrs=nMin/60;
UINT32 nSec=nTime%60;
INT8   szMin[6];
INT8   szHrs[6];
INT8   szSec[6];

  nMin%=60;
  if(nHrs>9) nHrs=0;
  SelectObject(hBmpDC, tSkin.hBmpMainTimeFont);
  //convert it all to text
  sprintf(szMin, "%02i", nMin);
  sprintf(szHrs, "%01i", nHrs);
  sprintf(szSec, "%02i", nSec);
  //hours
  BitBlt(hDC, tSkin.tLocation[SKINTIMETEXT].x1 + (tSkin.tLocation[SKINTIMETEXT].w1 * 0), tSkin.tLocation[SKINTIMETEXT].y1, tSkin.tLocation[SKINTIMETEXT].w1, tSkin.tLocation[SKINTIMETEXT].h1, hBmpDC, ((szHrs[0]-'0') * tSkin.tLocation[SKINTIMETEXT].w1), 0, SRCCOPY);
  //colon
  BitBlt(hDC, tSkin.tLocation[SKINTIMETEXT].x1 + (tSkin.tLocation[SKINTIMETEXT].w1 * 1), tSkin.tLocation[SKINTIMETEXT].y1, tSkin.tLocation[SKINTIMETEXT].w1, tSkin.tLocation[SKINTIMETEXT].h1, hBmpDC, (10 * tSkin.tLocation[SKINTIMETEXT].w1), 0, SRCCOPY);
  //minutes
  BitBlt(hDC, tSkin.tLocation[SKINTIMETEXT].x1 + (tSkin.tLocation[SKINTIMETEXT].w1 * 2), tSkin.tLocation[SKINTIMETEXT].y1, tSkin.tLocation[SKINTIMETEXT].w1, tSkin.tLocation[SKINTIMETEXT].h1, hBmpDC, ((szMin[0]-'0') * tSkin.tLocation[SKINTIMETEXT].w1), 0, SRCCOPY);
  BitBlt(hDC, tSkin.tLocation[SKINTIMETEXT].x1 + (tSkin.tLocation[SKINTIMETEXT].w1 * 3), tSkin.tLocation[SKINTIMETEXT].y1, tSkin.tLocation[SKINTIMETEXT].w1, tSkin.tLocation[SKINTIMETEXT].h1, hBmpDC, ((szMin[1]-'0') * tSkin.tLocation[SKINTIMETEXT].w1), 0, SRCCOPY);
  //colon
  BitBlt(hDC, tSkin.tLocation[SKINTIMETEXT].x1 + (tSkin.tLocation[SKINTIMETEXT].w1 * 4), tSkin.tLocation[SKINTIMETEXT].y1, tSkin.tLocation[SKINTIMETEXT].w1, tSkin.tLocation[SKINTIMETEXT].h1, hBmpDC, (10 * tSkin.tLocation[SKINTIMETEXT].w1), 0, SRCCOPY);
  //seconds
  BitBlt(hDC, tSkin.tLocation[SKINTIMETEXT].x1 + (tSkin.tLocation[SKINTIMETEXT].w1 * 5), tSkin.tLocation[SKINTIMETEXT].y1, tSkin.tLocation[SKINTIMETEXT].w1, tSkin.tLocation[SKINTIMETEXT].h1, hBmpDC, ((szSec[0]-'0') * tSkin.tLocation[SKINTIMETEXT].w1), 0, SRCCOPY);
  BitBlt(hDC, tSkin.tLocation[SKINTIMETEXT].x1 + (tSkin.tLocation[SKINTIMETEXT].w1 * 6), tSkin.tLocation[SKINTIMETEXT].y1, tSkin.tLocation[SKINTIMETEXT].w1, tSkin.tLocation[SKINTIMETEXT].h1, hBmpDC, ((szSec[1]-'0') * tSkin.tLocation[SKINTIMETEXT].w1), 0, SRCCOPY);
}

//draw the title
void skinDrawSongTitle(HWND hWnd, INT8 *szTitle)
{
UINT32 i, j;

  SelectObject(hBmpDC, tSkin.hBmpMainTextFont);
  i=strlen(szTitle);
  //if the text is longer than the space available, truncate it
  if(i>tSkin.tLocation[SKINTITLETEXT].x2) i=tSkin.tLocation[SKINTITLETEXT].x2;
  //draw it!
  for(j=0; j<i; j++)
    BitBlt(hDC, tSkin.tLocation[SKINTITLETEXT].x1 + (tSkin.tLocation[SKINTITLETEXT].w1 * j), tSkin.tLocation[SKINTITLETEXT].y1, tSkin.tLocation[SKINTITLETEXT].w1, tSkin.tLocation[SKINTITLETEXT].h1, hBmpDC, ((szTitle[j]-' ') * tSkin.tLocation[SKINTITLETEXT].w1), 0, SRCCOPY);

}

//draw the buttons
void skinDrawButton(HWND hWnd, UINT32 nButton, UINT32 nState)
{
  if(nState){
    SelectObject(hBmpDC, tSkin.hBmpMainDown);
    BitBlt(hDC, tSkin.tLocation[nButton].x1, tSkin.tLocation[nButton].y1, tSkin.tLocation[nButton].w1, tSkin.tLocation[nButton].h1, hBmpDC, tSkin.tLocation[nButton].x2, tSkin.tLocation[nButton].y2, SRCCOPY);
  }
}

//draw the volume part of the skin
void skinDrawVolume(HWND hWnd, UINT32 nVolume)
{
  //draw the volume slider
  SelectObject(hBmpDC, tSkin.hBmpMainDown);
  //a solid bar
  if(tSkin.tLocation[SKINVOLUMESLIDER].a==0){
    //horizontal
    if(tSkin.tLocation[SKINVOLUMESLIDER].b==0){
      //convert the volume to a location
      nVolume=(tSkin.tLocation[SKINVOLUMESLIDER].w1 * nVolume) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINVOLUMESLIDER].x1, tSkin.tLocation[SKINVOLUMESLIDER].y1, nVolume, tSkin.tLocation[SKINVOLUMESLIDER].h1, hBmpDC, tSkin.tLocation[SKINVOLUMESLIDER].x1, tSkin.tLocation[SKINVOLUMESLIDER].y1, SRCCOPY);
    }
    //vertical
    else if(tSkin.tLocation[SKINVOLUMESLIDER].b==1){
      //convert the volume to a location
      nVolume=tSkin.tLocation[SKINVOLUMESLIDER].h1 - ((tSkin.tLocation[SKINVOLUMESLIDER].h1 * nVolume) / 100);
      BitBlt(hDC, tSkin.tLocation[SKINVOLUMESLIDER].x1, tSkin.tLocation[SKINVOLUMESLIDER].y1 + nVolume, tSkin.tLocation[SKINVOLUMESLIDER].w1, tSkin.tLocation[SKINVOLUMESLIDER].h1 - nVolume, hBmpDC, tSkin.tLocation[SKINVOLUMESLIDER].x1, tSkin.tLocation[SKINVOLUMESLIDER].y1 + nVolume, SRCCOPY);
    }
  }

  //a slider
  else if(tSkin.tLocation[SKINVOLUMESLIDER].a==1){
    //horizontal
    if(tSkin.tLocation[SKINVOLUMESLIDER].b==0){
      //convert the volume to a location
      nVolume=(tSkin.tLocation[SKINVOLUMESLIDER].w1 * nVolume) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINVOLUMESLIDER].x1 + nVolume - (tSkin.tLocation[SKINVOLUMESLIDER].w2 / 2), tSkin.tLocation[SKINVOLUMESLIDER].y1, tSkin.tLocation[SKINVOLUMESLIDER].w2, tSkin.tLocation[SKINVOLUMESLIDER].h2, hBmpDC, tSkin.tLocation[SKINVOLUMESLIDER].x2, tSkin.tLocation[SKINVOLUMESLIDER].y2, SRCCOPY);
    }
    //vertical
    else if(tSkin.tLocation[SKINVOLUMESLIDER].b==1){
      //convert the volume to a location
      nVolume=tSkin.tLocation[SKINVOLUMESLIDER].h1 - ((tSkin.tLocation[SKINVOLUMESLIDER].h1 * nVolume) / 100);
      BitBlt(hDC, tSkin.tLocation[SKINVOLUMESLIDER].x1, tSkin.tLocation[SKINVOLUMESLIDER].y1 + tSkin.tLocation[SKINVOLUMESLIDER].h1 - nVolume - (tSkin.tLocation[SKINVOLUMESLIDER].h2 / 2), tSkin.tLocation[SKINVOLUMESLIDER].w2, tSkin.tLocation[SKINVOLUMESLIDER].h2, hBmpDC, tSkin.tLocation[SKINVOLUMESLIDER].x2, tSkin.tLocation[SKINVOLUMESLIDER].y2, SRCCOPY);
    }
  }
}

//draw the pan part of the skin
void skinDrawPan(HWND hWnd, INT32 nPan)
{
INT8 nW;

  nW=tSkin.tLocation[SKINPAN].w1 / 2;

  //draw the volume slider
  SelectObject(hBmpDC, tSkin.hBmpMainDown);
  //a solid bar
  if(tSkin.tLocation[SKINPAN].a==0){
    //horizontal
    if(tSkin.tLocation[SKINPAN].b==0){
      //convert the pan to a location
      nPan=(nW * nPan) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINPAN].x1 + nW, tSkin.tLocation[SKINPAN].y1, nPan, tSkin.tLocation[SKINPAN].h1, hBmpDC, tSkin.tLocation[SKINPAN].x1, tSkin.tLocation[SKINPAN].y1, SRCCOPY);
    }
    //vertical
    else if(tSkin.tLocation[SKINPAN].b==1){
      //is anyone really gonna do a up/down pan
    }
  }
  //a slider
  else if(tSkin.tLocation[SKINPAN].a==1){
    //horizontal
    if(tSkin.tLocation[SKINPAN].b==0){
      //convert the pan to a location
      nPan=(nW * nPan) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINPAN].x1 + nW + nPan - (tSkin.tLocation[SKINPAN].w2 / 2), tSkin.tLocation[SKINPAN].y1, tSkin.tLocation[SKINPAN].w2, tSkin.tLocation[SKINPAN].h2, hBmpDC, tSkin.tLocation[SKINPAN].x2, tSkin.tLocation[SKINPAN].y2, SRCCOPY);
    }
    //vertical
    else if(tSkin.tLocation[SKINPAN].b==1){
      //is anyone really gonna do a up/down pan
    }
  }
}

//draw the gain part of the skin
void skinDrawGain(HWND hWnd, INT32 nGain)
{
INT8 nW, nH;

  nW=tSkin.tLocation[SKINGAIN].w1 / 2;
  nH=tSkin.tLocation[SKINGAIN].h1 / 2;

  //draw the volume slider
  SelectObject(hBmpDC, tSkin.hBmpMainDown);
  //a solid bar
  if(tSkin.tLocation[SKINGAIN].a==0){
    //horizontal
    if(tSkin.tLocation[SKINGAIN].b==0){
      //convert the gain to a location
      nGain=(nW * nGain) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINGAIN].x1 + nW, tSkin.tLocation[SKINGAIN].y1, nGain, tSkin.tLocation[SKINGAIN].h1, hBmpDC, tSkin.tLocation[SKINGAIN].x1, tSkin.tLocation[SKINGAIN].y1, SRCCOPY);
    }
    //vertical
    else if(tSkin.tLocation[SKINGAIN].b==1){
      nGain=(nH * nGain) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINGAIN].x1, tSkin.tLocation[SKINGAIN].y1 + nH - nGain, tSkin.tLocation[SKINGAIN].w1, nH + nGain, hBmpDC, tSkin.tLocation[SKINGAIN].x1, tSkin.tLocation[SKINGAIN].y1 + nH - nGain, SRCCOPY);
    }
  }
  //a slider
  else if(tSkin.tLocation[SKINGAIN].a==1){
    //horizontal
    if(tSkin.tLocation[SKINGAIN].b==0){
      //convert the gain to a location
      nGain=(nW * nGain) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINGAIN].x1 + nW + nGain - (tSkin.tLocation[SKINGAIN].w2 / 2), tSkin.tLocation[SKINGAIN].y1, tSkin.tLocation[SKINGAIN].w2, tSkin.tLocation[SKINGAIN].h2, hBmpDC, tSkin.tLocation[SKINGAIN].x2, tSkin.tLocation[SKINGAIN].y2, SRCCOPY);
    }
    //vertical
    else if(tSkin.tLocation[SKINGAIN].b==1){
      nGain=(nH * nGain) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINGAIN].x1, tSkin.tLocation[SKINGAIN].y1 + nH - nGain - (tSkin.tLocation[SKINGAIN].h2 / 2), tSkin.tLocation[SKINGAIN].w2, tSkin.tLocation[SKINGAIN].h2, hBmpDC, tSkin.tLocation[SKINGAIN].x2, tSkin.tLocation[SKINGAIN].y2, SRCCOPY);
    }
  }
}

//draw the equalizers
void skinDrawEqualizer(HWND hWnd, UINT32 nEq, UINT32 nEqualizer)
{
  //draw the equalizer slider
  SelectObject(hBmpDC, tSkin.hBmpMainDown);
  //a solid bar
  if(tSkin.tLocation[SKINEQUALIZER1 + nEq].a==0){
    //horizontal
    if(tSkin.tLocation[SKINEQUALIZER1 + nEq].b==0){
      //convert the equalizer to a location
      nEqualizer=(tSkin.tLocation[SKINEQUALIZER1 + nEq].w1 * nEqualizer) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINEQUALIZER1 + nEq].x1, tSkin.tLocation[SKINEQUALIZER1 + nEq].y1, nEqualizer, tSkin.tLocation[SKINEQUALIZER1 + nEq].h1, hBmpDC, tSkin.tLocation[SKINEQUALIZER1 + nEq].x1, tSkin.tLocation[SKINEQUALIZER1 + nEq].y1, SRCCOPY);
    }
    //vertical
    else if(tSkin.tLocation[SKINEQUALIZER1 + nEq].b==1){
      //convert the equalizer to a location
      nEqualizer=tSkin.tLocation[SKINEQUALIZER1 + nEq].h1 - ((tSkin.tLocation[SKINEQUALIZER1 + nEq].h1 * nEqualizer) / 100);
      BitBlt(hDC, tSkin.tLocation[SKINEQUALIZER1 + nEq].x1, tSkin.tLocation[SKINEQUALIZER1 + nEq].y1 + nEqualizer, tSkin.tLocation[SKINEQUALIZER1 + nEq].w1, tSkin.tLocation[SKINEQUALIZER1 + nEq].h1 - nEqualizer, hBmpDC, tSkin.tLocation[SKINEQUALIZER1 + nEq].x1, tSkin.tLocation[SKINEQUALIZER1 + nEq].y1 + nEqualizer, SRCCOPY);
    }
  }
  //a slider
  else if(tSkin.tLocation[SKINEQUALIZER1 + nEq].a==1){
    //horizontal
    if(tSkin.tLocation[SKINEQUALIZER1 + nEq].b==0){
      //convert the equalizer to a location
      nEqualizer=(tSkin.tLocation[SKINEQUALIZER1 + nEq].w1 * nEqualizer) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINEQUALIZER1 + nEq].x1, tSkin.tLocation[SKINEQUALIZER1 + nEq].y1, nEqualizer, tSkin.tLocation[SKINEQUALIZER1 + nEq].h1, hBmpDC, tSkin.tLocation[SKINEQUALIZER1 + nEq].x1, tSkin.tLocation[SKINEQUALIZER1 + nEq].y1, SRCCOPY);
    }
    //vertical
    else if(tSkin.tLocation[SKINEQUALIZER1 + nEq].b==1){
      //convert the equalizer to a location
      nEqualizer=(tSkin.tLocation[SKINEQUALIZER1 + nEq].h1 * nEqualizer) / 100;
      BitBlt(hDC, tSkin.tLocation[SKINEQUALIZER1 + nEq].x1, tSkin.tLocation[SKINEQUALIZER1 + nEq].y1 + tSkin.tLocation[SKINEQUALIZER1 + nEq].h1 - nEqualizer - (tSkin.tLocation[SKINEQUALIZER1 + nEq].h2 / 2), tSkin.tLocation[SKINEQUALIZER1 + nEq].w2, tSkin.tLocation[SKINEQUALIZER1 + nEq].h2, hBmpDC, tSkin.tLocation[SKINEQUALIZER1 + nEq].x2, tSkin.tLocation[SKINEQUALIZER1 + nEq].y2, SRCCOPY);
    }
  }
}

//check to see if a point is inside a location
UINT32 skinCheckHit(INT32 nX, INT32 nY, UINT32 nLocation)
{
  //volume slider
  if(nLocation==SKINVOLUMESLIDER){
    //horizontal
    if(tSkin.tLocation[nLocation].b==0){
      //slider
      if(tSkin.tLocation[nLocation].a==1){
        if((nX >= tSkin.tLocation[nLocation].x1 - (tSkin.tLocation[nLocation].w2 / 2)) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1 + (tSkin.tLocation[nLocation].w2 / 2)) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
      //solid bar
      else{
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
    }
    //vertical
    if(tSkin.tLocation[nLocation].b==1){
      //slider
      if(tSkin.tLocation[nLocation].a==1){
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1 - (tSkin.tLocation[nLocation].h2 / 2)) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1 + (tSkin.tLocation[nLocation].h2 / 2))) return(1);
      }
      //solid bar
      else{
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
    }
  }
  //pan slider
  if(nLocation==SKINPAN){
    //horizontal
    if(tSkin.tLocation[nLocation].b==0){
      //slider
      if(tSkin.tLocation[nLocation].a==1){
        if((nX >= tSkin.tLocation[nLocation].x1 - (tSkin.tLocation[nLocation].w2 / 2)) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1 + (tSkin.tLocation[nLocation].w2 / 2)) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
      //solid bar
      else{
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
    }
    //vertical
    if(tSkin.tLocation[nLocation].b==1){
      //slider
      if(tSkin.tLocation[nLocation].a==1){
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1 - (tSkin.tLocation[nLocation].h2 / 2)) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1 + (tSkin.tLocation[nLocation].h2 / 2))) return(1);
      }
      //solid bar
      else{
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
    }
  }
  //equalizers
  if(nLocation>=SKINEQUALIZER1 && nLocation<=SKINEQUALIZER8){
    //horizontal
    if(tSkin.tLocation[nLocation].b==0){
      //slider
      if(tSkin.tLocation[nLocation].a==1){
        if((nX >= tSkin.tLocation[nLocation].x1 - (tSkin.tLocation[nLocation].w2 / 2)) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1 + (tSkin.tLocation[nLocation].w2 / 2)) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
      //solid bar
      else{
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
    }
    //vertical
    if(tSkin.tLocation[nLocation].b==1){
      //slider
      if(tSkin.tLocation[nLocation].a==1){
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1 - (tSkin.tLocation[nLocation].h2 / 2)) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1 + (tSkin.tLocation[nLocation].h2 / 2))) return(1);
      }
      //solid bar
      else{
        if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
      }
    }
  }
  //everything else is just normal
  else{
    if((nX >= tSkin.tLocation[nLocation].x1) && (nY >= tSkin.tLocation[nLocation].y1) && (nX < tSkin.tLocation[nLocation].x1 + tSkin.tLocation[nLocation].w1) && (nY < tSkin.tLocation[nLocation].y1 + tSkin.tLocation[nLocation].h1)) return(1);
  }
  return(0);
}

//return the space available for the title
UINT32 skinCheckTitleLength()
{
  return(tSkin.tLocation[SKINTITLETEXT].x2);
}

//returns the volume percentage based on the mouse location
UINT32 skinCheckVolume(INT32 nX, INT32 nY)
{
UINT32 nVolume=0;

  //horizontal
  if(tSkin.tLocation[SKINVOLUMESLIDER].b==0){
    if(nX<=tSkin.tLocation[SKINVOLUMESLIDER].x1) nVolume=0;
    else if(nX>=tSkin.tLocation[SKINVOLUMESLIDER].x1 + tSkin.tLocation[SKINVOLUMESLIDER].w1) nVolume=100;
    else{
      nVolume=nX - tSkin.tLocation[SKINVOLUMESLIDER].x1;
      nVolume=(nVolume * 100) / tSkin.tLocation[SKINVOLUMESLIDER].w1;
    }
  }
  //vertical
  else if(tSkin.tLocation[SKINVOLUMESLIDER].b==1){
    if(nY<=tSkin.tLocation[SKINVOLUMESLIDER].y1) nVolume=100;
    else if(nY>=tSkin.tLocation[SKINVOLUMESLIDER].y1 + tSkin.tLocation[SKINVOLUMESLIDER].h1) nVolume=0;
    else{
      nVolume=(tSkin.tLocation[SKINVOLUMESLIDER].y1 + tSkin.tLocation[SKINVOLUMESLIDER].h1) - nY;
      nVolume=(nVolume * 100) / tSkin.tLocation[SKINVOLUMESLIDER].h1;
    }
  }
  //return the volume percentage
  return(nVolume);
}

//returns the pan percentage based on the mouse location
INT32 skinCheckPan(INT32 nX, INT32 nY)
{
INT32 nPan=0;
UINT32 nW;

  //horizontal
  if(tSkin.tLocation[SKINPAN].b==0){
    if(nX<=tSkin.tLocation[SKINPAN].x1) nPan=-100;
    else if(nX>=tSkin.tLocation[SKINPAN].x1 + tSkin.tLocation[SKINPAN].w1) nPan=100;
    else{
      nW=tSkin.tLocation[SKINPAN].w1 / 2;
      if(nX - tSkin.tLocation[SKINPAN].x1 < nW){
        nPan=nX - tSkin.tLocation[SKINPAN].x1;
        nPan=(nPan * 100) / nW;
        nPan=-(100 - nPan);
      }
      else if(nX - tSkin.tLocation[SKINPAN].x1 > nW){
        nPan=nX - tSkin.tLocation[SKINPAN].x1 - nW;
        nPan=(nPan * 100) / nW;
      }
      else{
        nPan=0;
      }
    }
  }
  //vertical
  else if(tSkin.tLocation[SKINPAN].b==1){
    //is anyone really gonna do a up/down pan
  }
  //return the volume percentage
  return(nPan);
}

//returns the gain percentage based on the mouse location
INT32 skinCheckGain(INT32 nX, INT32 nY)
{
INT32 nPan=0;
UINT32 nW, nH;

  nW=tSkin.tLocation[SKINGAIN].w1 / 2;
  nH=tSkin.tLocation[SKINGAIN].h1 / 2;
  //horizontal
  if(tSkin.tLocation[SKINGAIN].b==0){
    if(nX<=tSkin.tLocation[SKINGAIN].x1) nPan=-100;
    else if(nX>=tSkin.tLocation[SKINGAIN].x1 + tSkin.tLocation[SKINGAIN].w1) nPan=100;
    else{
      if(nX - tSkin.tLocation[SKINGAIN].x1 < nW){
        nPan=nX - tSkin.tLocation[SKINGAIN].x1;
        nPan=(nPan * 100) / nW;
        nPan=-(100 - nPan);
      }
      else if(nX - tSkin.tLocation[SKINGAIN].x1 > nW){
        nPan=nX - tSkin.tLocation[SKINGAIN].x1 - nW;
        nPan=(nPan * 100) / nW;
      }
      else{
        nPan=0;
      }
    }
  }
  //vertical
  else if(tSkin.tLocation[SKINGAIN].b==1){

    if(nY<=tSkin.tLocation[SKINGAIN].y1) nPan=100;
    else if(nY>=tSkin.tLocation[SKINGAIN].y1 + tSkin.tLocation[SKINGAIN].h1) nPan=-100;
    else{
      if(nY <= tSkin.tLocation[SKINGAIN].y1 + nH){
        nPan=tSkin.tLocation[SKINGAIN].y1 + nH - nY;
        nPan=(nPan * 100) / nH;
      }
      else if(nY >= tSkin.tLocation[SKINGAIN].y1 + nH){
        nPan=nY - (tSkin.tLocation[SKINGAIN].y1 + nH);
//        nPan=(nPan * 100) / nH;
//        nPan=-(100 - nPan);
        nPan=-((nPan * 100) / nH);
      }
      else{
        nPan=0;
      }
    }
  
  }
  //return the volume percentage
  return(nPan);
}

//returns the equalizer percentage based on the mouse location
UINT32 skinCheckEqualizer(INT32 nX, INT32 nY, UINT32 nEq)
{
UINT32 nEqualizer=0;

  //horizontal
  if(tSkin.tLocation[SKINEQUALIZER1 + nEq].b==0){
    if(nX<=tSkin.tLocation[SKINEQUALIZER1 + nEq].x1) nEqualizer=0;
    else if(nX>=tSkin.tLocation[SKINEQUALIZER1 + nEq].x1 + tSkin.tLocation[SKINEQUALIZER1 + nEq].w1) nEqualizer=100;
    else{
      nEqualizer=nX - tSkin.tLocation[SKINEQUALIZER1 + nEq].x1;
      nEqualizer=(nEqualizer * 100) / tSkin.tLocation[SKINEQUALIZER1 + nEq].w1;
    }
  }
  //vertical
  else if(tSkin.tLocation[SKINEQUALIZER1 + nEq].b==1){
    if(nY<=tSkin.tLocation[SKINEQUALIZER1 + nEq].y1) nEqualizer=100;
    else if(nY>=tSkin.tLocation[SKINEQUALIZER1 + nEq].y1 + tSkin.tLocation[SKINEQUALIZER1 + nEq].h1) nEqualizer=0;
    else{
      nEqualizer=(tSkin.tLocation[SKINEQUALIZER1 + nEq].y1 + tSkin.tLocation[SKINEQUALIZER1 + nEq].h1) - nY;
      nEqualizer=(nEqualizer * 100) / tSkin.tLocation[SKINEQUALIZER1 + nEq].h1;
    }
  }
  //return the volume percentage
  return(nEqualizer);
}

//set up a region based on a bitmap and a transparency color and tolerance values
//this routine is based on code from Jean-Edouard Lachand Robert (http://www.geocities.com/Paris/LeftBank/1160/resume.htm)
HRGN skinBitmapToRegion(HBITMAP hBmp, COLORREF cTransparentColor, COLORREF cTolerance)
{
HRGN             hRgn=NULL, h=NULL;
HDC              hDC=NULL, hMemDC=NULL;
BITMAP           bm, bm32;
VOID            *pBits32=NULL;
HBITMAP          hBmp32=NULL, hOldBmp=NULL;
BITMAPINFOHEADER RGB32BITSBITMAPINFO={sizeof(BITMAPINFOHEADER), 0, 0, 1, 32, BI_RGB, 0, 0, 0, 0, 0};
UINT32           maxRects=100;
HANDLE           hData;
RGNDATA         *pData;
RECT            *pRect;
UINT8            loR=GetRValue(cTransparentColor);
UINT8            loG=GetGValue(cTransparentColor);
UINT8            loB=GetBValue(cTransparentColor);
UINT8            hiR =min(0xff, loR + GetRValue(cTolerance));
UINT8            hiG =min(0xff, loG + GetGValue(cTolerance));
UINT8            hiB =min(0xff, loB + GetBValue(cTolerance));
UINT8           *p32, b;
INT32            y, x, x0, *p;

  if(hBmp){
    hMemDC=CreateCompatibleDC(NULL);
    if(hMemDC){
      GetObject(hBmp, sizeof(bm), &bm);
      RGB32BITSBITMAPINFO.biWidth = bm.bmWidth;
      RGB32BITSBITMAPINFO.biHeight = bm.bmHeight;
      hBmp32=CreateDIBSection(hMemDC, (BITMAPINFO *)&RGB32BITSBITMAPINFO, DIB_RGB_COLORS, &pBits32, NULL, 0);
      if(hBmp32){
        hOldBmp=(HBITMAP) SelectObject(hMemDC, hBmp32);
        hDC=CreateCompatibleDC(hMemDC);
        if(hDC){
          GetObject(hBmp32, sizeof(bm32), &bm32);
          while(bm32.bmWidthBytes % 4) bm32.bmWidthBytes++;
          hOldBmp = (HBITMAP) SelectObject(hDC, hBmp);
          BitBlt(hMemDC, 0, 0, bm.bmWidth, bm.bmHeight, hDC, 0, 0, SRCCOPY);
          hData=GlobalAlloc(GMEM_MOVEABLE, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects));
          pData = (RGNDATA *) GlobalLock(hData);
          pData->rdh.dwSize = sizeof(RGNDATAHEADER);
          pData->rdh.iType = RDH_RECTANGLES;
          pData->rdh.nCount = pData->rdh.nRgnSize = 0;
          SetRect(&pData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);
          p32=(UINT8 *)bm32.bmBits + (bm32.bmHeight - 1) * bm32.bmWidthBytes;
          for(y=0; y<bm.bmHeight; y++){
            for(x=0; x<bm.bmWidth; x++){
              x0=x;
              p=(INT32 *)p32 + x;
              while(x < bm.bmWidth){
                b=GetRValue(*p);
                if(b >= loR && b <= hiR){
                  b=GetGValue(*p);
                  if(b >= loG && b <= hiG){
                    b=GetBValue(*p);
                    if(b >= loB && b <= hiB) break;
                  }
                }
                p++;
                x++;
              }
              if(x > x0){
                if(pData->rdh.nCount >= maxRects){
                  GlobalUnlock(hData);
                  maxRects += 100;
                  hData=GlobalReAlloc(hData, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), GMEM_MOVEABLE);
                  pData=(RGNDATA *)GlobalLock(hData);
                }
                pRect = (RECT *) & pData->Buffer;
                SetRect(&pRect[pData->rdh.nCount], x0, y, x, y + 1);
                if(x0 < pData->rdh.rcBound.left) pData->rdh.rcBound.left = x0;
                if(y < pData->rdh.rcBound.top) pData->rdh.rcBound.top = y;
                if(x > pData->rdh.rcBound.right) pData->rdh.rcBound.right = x;
                if(y + 1 > pData->rdh.rcBound.bottom) pData->rdh.rcBound.bottom = y + 1;
                pData->rdh.nCount++;
                if(pData->rdh.nCount==2000){
                  h=ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) *maxRects), pData);
                  if(hRgn){
                    CombineRgn(hRgn, hRgn, h, RGN_OR);
                    DeleteObject(h);
                  }
                  else hRgn=h;
                  pData->rdh.nCount = 0;
                  SetRect(&pData->rdh.rcBound, 0xffffffff, 0xffffffff, 0, 0);
                }
              }
            }
            p32-=bm32.bmWidthBytes;
          }
          h=ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pData);
          if(hRgn){
            CombineRgn(hRgn, hRgn, h, RGN_OR);
            DeleteObject(h);
          }
          else hRgn=h;
          GlobalUnlock(hData);
          GlobalFree(hData);
          SelectObject(hDC, hOldBmp);
          DeleteDC(hDC);
        }
        DeleteObject(SelectObject(hMemDC, hOldBmp));
      }
      DeleteDC(hMemDC);
    }
  }
  return(hRgn);
}

//parse the skin.ini file
UINT32 skinParse(INT8 *szFile)
{
UINT32  r;
INT8    szValues[32768];
INT8   *iPos;
UINT32  I;
INT8    szField[128];
INT32   x1, y1, w1, h1, x2, y2, w2, h2, a, b, c, d;

  //windows has a nice little built in routine that will read in INI files for us
  //all we have to do is parse the keys
  //we will do something very similar for playlists I think
  ZeroMemory(szValues, 32768);
  r=GetPrivateProfileSection("MERIDIAN PRIME SKIN", szValues, 32767, szFile);
  if(r==0) return(0);
  iPos=szValues;
  while(*iPos!=0){
    //convert '=' and ',' to a blank space for easier parsing
    I=0;
    while(I < strlen(iPos)){
	  if(iPos[I]=='=' || iPos[I]==',') iPos[I]=' ';
	  I++;
    }
    //read from the values to get the fields we need
    sscanf(iPos, "%s %d %d %d %d %d %d %d %d %d %d %d %d", &szField, &x1, &y1, &w1, &h1, &x2, &y2, &w2, &h2, &a, &b, &c, &d);
    //loop through possible fields and find which one we have
    for(I=0; I<SKINTRANSPARENTCOLOR; I++){
      if(stricmp(szField, tAssoc[I].szField)==0){
        //fill in the values regardless of the field
        tSkin.tLocation[I].x1=x1;
        tSkin.tLocation[I].y1=y1;
        tSkin.tLocation[I].w1=w1;
        tSkin.tLocation[I].h1=h1;
        tSkin.tLocation[I].x2=x2;
        tSkin.tLocation[I].y2=y2;
        tSkin.tLocation[I].w2=w2;
        tSkin.tLocation[I].h2=h2;
        tSkin.tLocation[I].a=a;
        tSkin.tLocation[I].b=b;
        tSkin.tLocation[I].c=c;
        tSkin.tLocation[I].d=d;
        break;
      }
    }
    if(stricmp(szField, "BmpMainUp")==0) strcpy(tSkin.szBmpMainUp, iPos + strlen(szField) + 1);
    if(stricmp(szField, "BmpMainDown")==0) strcpy(tSkin.szBmpMainDown, iPos + strlen(szField) + 1);
    if(stricmp(szField, "BmpMainTextFont")==0) strcpy(tSkin.szBmpMainTextFont, iPos + strlen(szField) + 1);
    if(stricmp(szField, "BmpMainTimeFont")==0) strcpy(tSkin.szBmpMainTimeFont, iPos + strlen(szField) + 1);
    if(stricmp(szField, "BmpMainTrackFont")==0) strcpy(tSkin.szBmpMainTrackFont, iPos + strlen(szField) + 1);
    if(stricmp(szField, "TransparentColor")==0) sscanf(iPos + strlen(szField)+1, "%x", &tSkin.cTransparent);
	iPos=iPos + strlen(iPos) + 1;
  }
  return(1);
}

//try to load a bitmap for the skin
UINT32 skinLoadBMP(HWND hWnd, INT8 *szFilename, HBITMAP *hBitmap, zipEntry *pEntries, INT32 nEntries)
{
INT32     i=0;
UINT8    *pMem=NULL;
INT8      szFile[256], szDir[256];
FILE     *fp;

  //get the temp path
  GetTempPath(256, szDir);
  //find the up bmp file
  for(i=0;i<nEntries;i++){
    if(!memicmp(pEntries[i].szName, szFilename, strlen(szFilename))) break;
  }
  if(i==nEntries){
    Message("Skin: Unable to locate bmp file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }
  //create the bmp file in the temp directory
  sprintf(szFile, "%s\\skin.bmp", szDir);
  fp=fopen(szFile, "w+b");;
  //load the bmp from the zip
  pMem=(UINT8 *)malloc(pEntries[i].nLen);
  zipLoad(pMem, pEntries[i].nLen, i);
  //write the file
  fwrite(pMem, 1, pEntries[i].nLen, fp);
  free(pMem);
  //close the bmp file
  fclose(fp);
  //load the image
  *hBitmap=(HBITMAP)LoadImage(NULL, szFile, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_LOADFROMFILE);
  //delete the temp file
  unlink(szFile);
  if(tSkin.hBmpMainUp==NULL){
    Message("Skin: Unable to load bmp file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }

  return(1);
}

//load a skin and set the transparency region
UINT32 skinLoad(HWND hWnd, INT8 *szFilename, UINT32 nX, UINT32 nY)
{
HRGN      hRgnMain;
zipEntry *pEntries=NULL;
INT32     nEntries=0;
INT8      szFile[256], szDir[256];
INT32     i=0;
UINT8    *pMem=NULL;
FILE     *fp;
BITMAP    bm;

  //save skin name
  strcpy(tSkin.szINI, szFilename);
  //clear skin struct
  ZeroMemory(&tSkin, sizeof(tSkin));
  //open the zipped skin file
  if(zipOpen(szFilename)==0){
    Message("Skin: Unable to open skin file.\n%s", szFilename);
    return(0);
  }
  zipGetList(&pEntries, &nEntries);
  //find the ini file
  for(i=0;i<nEntries;i++){
    if(!memicmp(pEntries[i].szName + (strlen(pEntries[i].szName)-4), ".ini", 4)) break;
  }
  if(i==nEntries){
    Message("Skin: Unable to locate ini file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }
  //get the temp path
  GetTempPath(256, szDir);
  //create the ini file in the temp directory
  sprintf(szFile, "%s\\skin.ini", szDir);
  fp=fopen(szFile, "w+b");
  //load the ini from the zip
  pMem=(UINT8 *)malloc(pEntries[i].nLen);
  zipLoad(pMem, pEntries[i].nLen, i);
  //write the file
  fwrite(pMem, 1, pEntries[i].nLen, fp);
  free(pMem);
  //close the ini file
  fclose(fp);
  //parse skin file
  if(skinParse(szFile)==0){
    Message("Skin: Unable to parse ini file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }
  //delete the file
  unlink(szFile);
  //load the MAIN UP skin
  if(skinLoadBMP(hWnd, tSkin.szBmpMainUp, &tSkin.hBmpMainUp, pEntries, nEntries)==0){
    Message("Skin: Unable to load UP bmp file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }
  GetObject(tSkin.hBmpMainUp, sizeof(bm), &bm);
  tSkin.nMainWidth=bm.bmWidth;
  tSkin.nMainHeight=bm.bmHeight;
  //load the MAIN DOWN skin
  if(skinLoadBMP(hWnd, tSkin.szBmpMainDown, &tSkin.hBmpMainDown, pEntries, nEntries)==0){
    Message("Skin: Unable to load DOWN bmp file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }
  //load the TEXTFONT skin
  if(skinLoadBMP(hWnd, tSkin.szBmpMainTextFont, &tSkin.hBmpMainTextFont, pEntries, nEntries)==0){
    Message("Skin: Unable to load TEXTFONT bmp file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }
  //load the TIMEFONT skin
  if(skinLoadBMP(hWnd, tSkin.szBmpMainTimeFont, &tSkin.hBmpMainTimeFont, pEntries, nEntries)==0){
    Message("Skin: Unable to load TIMEFONT bmp file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }
  //load the TRACKFONT skin
  if(skinLoadBMP(hWnd, tSkin.szBmpMainTrackFont, &tSkin.hBmpMainTrackFont, pEntries, nEntries)==0){
    Message("Skin: Unable to load TRACKFONT bmp file.");
    zipClose();
	zipFreeList(pEntries, nEntries);
    return(0);
  }
  //close the zipped skin
  zipClose();
  zipFreeList(pEntries, nEntries);
  //create the main transparency region
  hRgnMain=skinBitmapToRegion(tSkin.hBmpMainUp, tSkin.cTransparent, 0x000000);
  if(hRgnMain==NULL){
    Message("Skin: Unable to convert bitmap to region.");
    return(0);
  }
  //change the main window size based on bitmap size
  SetWindowPos(hWnd, 0, nX, nY, tSkin.nMainWidth, tSkin.nMainHeight, SWP_NOZORDER | SWP_SHOWWINDOW);
  //set the main region
  SetWindowRgn(hWnd, hRgnMain, TRUE);
  //create a back bitmap for double buffering
  hDC=GetDC(hWnd);
  hBackBuffer=CreateCompatibleBitmap(hDC, tSkin.nMainWidth, tSkin.nMainHeight);
  ReleaseDC(hWnd, hDC);
  if(hBackBuffer==NULL) return(0);
  return(1);
}

void skinFree()
{
  if(hBackBuffer) DeleteObject(hBackBuffer); hBackBuffer=NULL;
  if(tSkin.hBmpMainUp) DeleteObject(tSkin.hBmpMainUp); tSkin.hBmpMainUp=NULL;
  if(tSkin.hBmpMainDown) DeleteObject(tSkin.hBmpMainDown); tSkin.hBmpMainDown=NULL;
  if(tSkin.hBmpMainTextFont) DeleteObject(tSkin.hBmpMainTextFont); tSkin.hBmpMainTextFont=NULL;
  if(tSkin.hBmpMainTimeFont) DeleteObject(tSkin.hBmpMainTimeFont); tSkin.hBmpMainTimeFont=NULL;
  if(tSkin.hBmpMainTrackFont) DeleteObject(tSkin.hBmpMainTrackFont); tSkin.hBmpMainTrackFont=NULL;
}