/*
 * Meridian Prime
 * Copyright (C) 2002 Jason Cline
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <windows.h>
#include "global.h"
#include "zlib/unzip.h"

/*
 The contents below are a modified version of the zip interface from the
 FinalBurn project located at http://www.finalburn.com
*/
static unzFile Zip=NULL;
static int nCurrFile=0;

struct zipEntry{
  INT8   *szName;
  INT32   nLen;
  UINT32  nCrc;
};

//open a zip file
UINT32 zipOpen(INT8 *szFile)
{
  Zip=unzOpen(szFile);
  if(Zip==NULL) return(0);
  unzGoToFirstFile(Zip);
  nCurrFile=0;
  return(1);
}

//close a zip file
UINT32 zipClose()
{
  if(Zip!=NULL) unzClose(Zip);
  Zip=NULL;
  return(1);
}

UINT32 zipFreeList(struct zipEntry *pList, INT32 nListCount)
{
INT32 i;

  for(i=0;i<nListCount;i++)
    free(pList[i].szName);
  free(pList);
  return(1);
}

//get the contents of a zip file into an array of ZipEntrys
UINT32 zipGetList(struct zipEntry **pList, INT32 *pnListCount)
{
INT32 nRet=0;
INT32 nNextRet=0;
unz_global_info ZipGlobalInfo;
struct zipEntry *List=NULL;
INT32 nListLen=0;

  if(Zip==NULL) return(0);
  if(pList==NULL) return(0);
  ZeroMemory(&ZipGlobalInfo, sizeof(ZipGlobalInfo));
  unzGetGlobalInfo(Zip,&ZipGlobalInfo);
  nListLen=ZipGlobalInfo.number_entry;
  //make an array of file entries
  List=(struct zipEntry *)malloc(nListLen * sizeof(struct zipEntry));
  if(List==NULL){
    unzClose(Zip);
    return(0);
  }
  ZeroMemory(List, nListLen * sizeof(struct zipEntry));
  nRet=unzGoToFirstFile(Zip);
  if(nRet!=UNZ_OK){
    unzClose(Zip);
    return(0);
  }
  //step through all of the files, until we get to the end
  for(nCurrFile=0, nNextRet=UNZ_OK; nCurrFile<nListLen && nNextRet==UNZ_OK; nCurrFile++, nNextRet=unzGoToNextFile(Zip)){
    unz_file_info FileInfo;
    INT8 *szName=NULL;
    ZeroMemory(&FileInfo, sizeof(FileInfo));
    nRet=unzGetCurrentFileInfo(Zip, &FileInfo, NULL, 0, NULL, 0, NULL, 0);
    if(nRet!=UNZ_OK) continue;
    // Allocate space for the filename
    szName=(char *)malloc(FileInfo.size_filename+1); if (szName==NULL) continue;
    nRet=unzGetCurrentFileInfo(Zip,&FileInfo,szName,FileInfo.size_filename+1,NULL,0,NULL,0);

    List[nCurrFile].szName=szName;
    List[nCurrFile].nLen=FileInfo.uncompressed_size;
    List[nCurrFile].nCrc=FileInfo.crc;
  }
  // return the file list
  *pList=List;
  if(pnListCount!=NULL) *pnListCount=nListLen;
  unzGoToFirstFile(Zip);
  nCurrFile=0;
  return 0;
}

//load a file from a zip file
UINT32 zipLoad(UINT8 *Dest, INT32 nLen, INT32 nEntry)
{
UINT32 nRet=0;

  if(Zip==NULL) return(0);
  //start at first file
  nRet=unzGoToFirstFile(Zip);
  if(nRet!=UNZ_OK) return(0);
  nCurrFile=0;
  //step through to the file we need
  while(nCurrFile<nEntry){
    nRet=unzGoToNextFile(Zip);
    if (nRet!=UNZ_OK) return(0);
    nCurrFile++;
  }
  //open file
  nRet=unzOpenCurrentFile(Zip);
  if(nRet!=UNZ_OK) return(0);
  //read file
  nRet=unzReadCurrentFile(Zip, Dest, nLen);
  //close file
  nRet=unzCloseCurrentFile(Zip);
  if(nRet!=UNZ_OK) return(0);
  return(1);
}
